from dataclasses import dataclass
from shikimori.types.photo import PhotoExtended, Photo
from shikimori.types.animes import Anime
from typing import Any

from .manga import Manga
from shikimori.constants import SHIKIMORI_URL
from ..utils.filter import handle_none_data


@dataclass
class User:
    id: int
    nickname: str
    image: PhotoExtended
    avatar: str
    last_online_at: str
    url: str

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            id=data.get("id"),
            nickname=data.get("nickname"),
            image=PhotoExtended.from_dict(data.get("image")),
            avatar=data.get("avatar"),
            last_online_at=data.get("last_online_at"),
            url=data.get("url"),
        )


@dataclass
class ValueObj:
    name: Any
    value: Any

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(name=data["name"], value=data["value"])


@dataclass
class UserTitle:
    id: int
    grouped_id: str
    name: str
    size: int
    type: str

    @classmethod
    @handle_none_data
    def from_dict(cls, data):
        return cls(
            id=data.get("id"),
            grouped_id=data.get("grouped_id"),
            name=data.get("name"),
            size=data.get("size"),
            type=data.get("type"),
        )


@dataclass
class Statuses:
    animes: list[UserTitle]
    manga: list[UserTitle]

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            animes=[
                UserTitle.from_dict(anime_data) for anime_data in data.get("anime", [])
            ],
            manga=[
                UserTitle.from_dict(manga_data) for manga_data in data.get("manga", [])
            ],
        )


@dataclass
class Obj:
    anime: list[ValueObj]
    manga: list[ValueObj]

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            anime=[ValueObj.from_dict(obj_data) for obj_data in data.get("anime", [])],
            manga=[ValueObj.from_dict(obj_data) for obj_data in data.get("manga", [])],
        )


@dataclass
class Stats:
    statuses: Statuses
    full_statuses: Statuses
    scores: Obj
    types: Obj
    ratings: Obj
    has_anime: bool
    has_manga: bool
    genres: list
    studios: list
    studios: list
    publishers: list
    activity: list[ValueObj]

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            statuses=Statuses.from_dict(data.get("statuses")),
            full_statuses=Statuses.from_dict(data.get("full_statuses")),
            scores=Obj.from_dict(data.get("scores")),
            types=Obj.from_dict(data.get("types")),
            ratings=Obj.from_dict(data.get("ratings")),
            has_anime=data.get("has_anime?"),
            has_manga=data.get("has_manga?"),
            genres=data.get("genres", []),
            studios=data.get("studios", []),
            publishers=data.get("publishers", []),
            activity=[
                ValueObj.from_dict(obj_data) for obj_data in data.get("activity", [])
            ],
        )


@dataclass
class UserInfoInc(User):
    name: str | None
    sex: str | None
    full_years: int | None
    birth_on: Any | None
    locale: str

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            name=data.get("name"),
            sex=data.get("sex"),
            full_years=data.get("full_years"),
            birth_on=data.get("birth_on"),
            locale=data.get("locale"),
            id=data.get("id"),
            nickname=data.get("nickname"),
            image=PhotoExtended.from_dict(data.get("image")),
            avatar=data.get("avatar"),
            last_online_at=data.get("last_online_at"),
            url=data.get("url"),
        )

    @property
    def avatar_url(self) -> str:
        return f"{SHIKIMORI_URL}{self.avatar_url}"


@dataclass
class UserInfo(User):
    name: str | None
    sex: str | None
    full_years: int | None
    last_online: str
    website: str
    location: str | None
    banned: bool
    about: str
    about_html: str
    common_info: list[str]
    show_comments: bool
    in_friends: bool | None
    is_ignored: bool
    stats: Stats
    style_id: int

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            name=data.get("name"),
            sex=data.get("sex"),
            full_years=data.get("full_years"),
            last_online=data.get("last_online"),
            website=data.get("website"),
            location=data.get("location"),
            banned=data.get("banned"),
            about=data.get("about"),
            about_html=data.get("about_html"),
            common_info=data.get("common_info", []),
            show_comments=data.get("show_comments"),
            in_friends=data.get("in_friends"),
            is_ignored=data.get("is_ignored"),
            stats=Stats.from_dict(data.get("stats")),
            style_id=data.get("style_id"),
            id=data["id"],
            nickname=data["nickname"],
            image=PhotoExtended.from_dict(data.get("image")),
            avatar=data["avatar"],
            last_online_at=data["last_online_at"],
            url=data["url"],
        )


@dataclass
class Rate:
    anime: Anime | None
    manga: Manga | None
    user: User
    id: int
    score: int
    status: str
    text: str
    episodes: int
    chapters: int
    volumes: int
    text_html: str
    rewatches: int

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            anime=Anime.from_dict(data.get("anime")) if data.get("anime") else None,
            manga=Manga.from_dict(data.get("manga")) if data.get("manga") else None,
            user=User.from_dict(data["user"]),
            id=data["id"],
            score=data["score"],
            status=data["status"],
            text=data["text"],
            episodes=data["episodes"],
            chapters=data["chapters"],
            volumes=data["volumes"],
            text_html=data["text_html"],
            rewatches=data["rewatches"],
        )


@dataclass
class FavouritesObj:
    id: int
    name: str
    russian: str
    image: str
    url: None

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            id=data["id"],
            name=data["name"],
            russian=data["russian"],
            image=data["image"],
            url=None,
        )


@dataclass
class Favourites:
    animes: list[FavouritesObj]
    mangas: list[FavouritesObj]
    ranobe: list[FavouritesObj]
    characters: list[FavouritesObj]
    mangakas: list[FavouritesObj]
    seyu: list[FavouritesObj]
    producers: list[FavouritesObj]

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            animes=[
                FavouritesObj.from_dict(obj_data) for obj_data in data.get("animes", [])
            ],
            mangas=[
                FavouritesObj.from_dict(obj_data) for obj_data in data.get("mangas", [])
            ],
            ranobe=[
                FavouritesObj.from_dict(obj_data) for obj_data in data.get("ranobe", [])
            ],
            characters=[
                FavouritesObj.from_dict(obj_data)
                for obj_data in data.get("characters", [])
            ],
            mangakas=[
                FavouritesObj.from_dict(obj_data)
                for obj_data in data.get("mangakas", [])
            ],
            seyu=[
                FavouritesObj.from_dict(obj_data) for obj_data in data.get("seyu", [])
            ],
            producers=[
                FavouritesObj.from_dict(obj_data)
                for obj_data in data.get("producers", [])
            ],
        )


@dataclass
class UnreadMessages:
    messages: int
    news: int
    notifications: int

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            messages=data.get("messages"),
            news=data.get("news"),
            notifications=data.get("notifications"),
        )


@dataclass
class TitleHistory:
    id: int
    name: str
    russian: str
    image: Photo
    url: str
    kind: str
    score: str
    status: str
    aired_on: str
    released_on: str
    volumes: int | None
    chapters: int | None
    episodes: int | None
    episodes_aired: int | None

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            volumes=data.get("volumes"),
            chapters=data.get("chapters"),
            episodes=data.get("episodes"),
            episodes_aired=data.get("episodes_aired"),
            id=data["id"],
            name=data["name"],
            russian=data["russian"],
            image=Photo.from_dict(data["image"]),
            url=data["url"],
            kind=data["kind"],
            score=data["score"],
            status=data["status"],
            aired_on=data["aired_on"],
            released_on=data["released_on"],
        )


@dataclass
class HistoryObj:
    id: int
    description: str
    target: TitleHistory | None

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            id=data["id"],
            description=data["description"],
            target=(
                TitleHistory.from_dict(data.get("target"))
                if data.get("target")
                else None
            ),
        )



from dataclasses import dataclass
from shikimori.utils.filter import handle_none_data


@dataclass
class AbuseRequest:
    kind: str
    value: bool
    affected_ids: list[int]

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            kind=data.get("kind"),
            value=data.get("value"),
            affected_ids=data.get("affected_ids"),
        )



from dataclasses import dataclass

from shikimori.utils.filter import handle_none_data


@dataclass
class Video:
    id: int
    url: str
    image_url: str
    player_url: str
    name: str
    kind: str
    hosting: str

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            id=data.get("id"),
            url=data.get("url"),
            image_url=data.get("image_url"),
            player_url=data.get("player_url"),
            name=data.get("name"),
            kind=data.get("kind"),
            hosting=data.get("hosting"),
        )



import logging
from typing import List

from shikimori.types.dialog import Dialog
from shikimori.types.message import MessageInfo
from .base import BaseEndpoint
from ..exceptions import RequestError

logger = logging.getLogger(__name__)


class DialogsEndpoint(BaseEndpoint):
    async def list(self) -> List[Dialog] | RequestError:
        """
        List dialogs.

        Requires messages oauth scope.
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/dialogs",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return [Dialog.from_dict(dialog) for dialog in response]

        logger.debug(
            f"Bad Request(list): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def ById(self, id: str) -> List[MessageInfo] | RequestError:
        """
        Show a dialog.
        Requires messages oauth scope.

        :param id: like user_{id}.
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/dialogs/{id}",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return [MessageInfo.from_dict(msg) for msg in response]

        logger.debug(
            f"Bad Request(ById): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def delete(self, id: str) -> str | RequestError:
        """
        Destroy a dialog.
        Requires messages oauth scope.

        :param id: like user_{id}.
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/dialogs/{id}",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return response["notice"]

        logger.debug(
            f"Bad Request(delete): status - {response.status_code}: info - {str(response)}"
        )

        return response



from dataclasses import dataclass

from shikimori.utils.filter import handle_none_data


@dataclass
class MangaConstant:
    kind: list[str]
    status: list[str]

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(kind=data.get("kind", []), status=data.get("status", []))


@dataclass
class AnimeConstant:
    kind: list[str]
    status: list[str]

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(kind=data.get("kind", []), status=data.get("status", []))


@dataclass
class UserRateConstant:
    status: list[str]

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(status=data.get("status", []))


@dataclass
class ClubConstant:
    join_policy: list[str]
    comment_policy: list[str]
    image_upload_policy: list[str]

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):

        return cls(
            join_policy=data.get("join_policy", []),
            comment_policy=data.get("comment_policy", []),
            image_upload_policy=data.get("image_upload_policy", []),
        )


@dataclass
class SmileConstant:
    bbcode: str
    path: str

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(bbcode=data.get("bbcode"), path=data.get("path"))



import logging

from .base import BaseEndpoint
from ..exceptions import RequestError

logger = logging.getLogger(__name__)


class FriendEndpoint(BaseEndpoint):
    async def add(self, id: int) -> str | RequestError:
        """
        Create a friend.

        Requires friends oauth scope.

        :param id: must be a number.
        """
        response = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/api/friends/{id}",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return response["notice"]

        logger.debug(
            f"Bad Request(add): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def delete(self, id: int) -> str | RequestError:
        """
        Destroy a friend.

        Requires friends oauth scope.

        :param id: must be a number.
        """
        response = await self._request.make_request(
            "DELETE",
            url=f"{self._base_url}/api/friends/{id}",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return response["notice"]

        logger.debug(
            f"Bad Request(delete): status - {response.status_code}: info - {str(response)}"
        )

        return response



from dataclasses import dataclass

from .manga import Manga
from .photo import Photo
from .screenshots import ScreenShot
from .studios import Studio
from .user_rates import UserRateBrief
from .videos import Video
from ..utils.filter import handle_none_data
from .genres import GenreExtended


@dataclass
class Anime:
    id: int
    name: str
    russian: str
    image: Photo
    url: str
    kind: str
    score: str
    status: str
    aired_on: str
    released_on: str
    episodes: int
    episodes_aired: int

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            id=data.get("id"),
            name=data.get("name"),
            russian=data.get("russian"),
            image=Photo.from_dict(data.get("image")),
            url=data.get("url"),
            kind=data.get("kind"),
            score=data.get("score"),
            status=data.get("status"),
            episodes=data.get("episodes"),
            episodes_aired=data.get("episodes_aired"),
            aired_on=data.get("aired_on"),
            released_on=data.get("released_on"),
        )


@dataclass
class AnimeInfo(Anime):
    english: list[str] | list[None]
    japanese: list[str] | list[None]
    synonyms: list[str] | list[None]
    license_name_ru: str | None
    duration: int
    description: str
    description_html: str
    description_source: str | None
    franchise: str | None
    anons: bool
    ongoing: bool
    favoured: bool
    thread_id: int
    rating: str
    topic_id: int
    myanimelist_id: int
    rates_scores_stats: list[dict]
    rates_statuses_stats: list[dict]
    updated_at: str
    next_episode_at: str | None
    fansubbers: list[dict]
    fandubbers: list[dict]
    licensors: list[dict]
    genres: list[GenreExtended]
    studios: list[Studio]
    videos: list[Video]
    screenshots: list[ScreenShot]
    user_rate: UserRateBrief

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            id=data.get("id"),
            name=data.get("name"),
            russian=data.get("russian"),
            image=data.get("image"),
            url=data.get("url"),
            kind=data.get("kind"),
            score=data.get("score"),
            status=data.get("status"),
            episodes=data.get("episodes"),
            episodes_aired=data.get("episodes_aired"),
            aired_on=data.get("aired_on"),
            released_on=data.get("released_on"),
            rating=data.get("rating"),
            english=data.get("english"),
            japanese=data.get("japanese"),
            synonyms=data.get("synonyms"),
            license_name_ru=data.get("license_name_ru"),
            duration=data.get("duration"),
            description=data.get("description"),
            description_html=data.get("description_html"),
            description_source=data.get("description_source"),
            franchise=data.get("franchise"),
            favoured=data.get("favoured"),
            anons=data.get("anons"),
            ongoing=data.get("ongoing"),
            thread_id=data.get("thread_id"),
            topic_id=data.get("topic_id"),
            myanimelist_id=data.get("myanimelist_id"),
            rates_scores_stats=data.get("rates_scores_stats"),
            rates_statuses_stats=data.get("rates_statuses_stats"),
            updated_at=data.get("updated_at"),
            next_episode_at=data.get("next_episode_at"),
            fansubbers=data.get("fansubbers"),
            fandubbers=data.get("fandubbers"),
            licensors=data.get("licensors"),
            genres=[GenreExtended.from_dict(genre) for genre in data.get("genres", [])],
            screenshots=[ScreenShot.from_dict(s) for s in data.get("screenshots", [])],
            studios=[Studio.from_dict(s) for s in data.get("studios", [])],
            videos=[Video.from_dict(v) for v in data.get("videos", [])],
            user_rate=(
                UserRateBrief.from_dict(data.get("user_rate"))
                if data.get("user_rate")
                else None
            ),
        )


@dataclass
class Relation:
    relation: str
    relation_russian: str
    anime: Anime | None
    manga: Manga | None

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            relation=data.get("relation"),
            relation_russian=data.get("relation_russian"),
            anime=Anime.from_dict(anime) if (anime := data.get("anime")) else None,
            manga=Manga.from_dict(manga) if (manga := data.get("manga")) else None,
        )



import logging
from typing import List

from shikimori.types.comment import Comment
from .base import BaseEndpoint
from ..exceptions import RequestError
from ..utils.filter import filter_none_parameters

logger = logging.getLogger(__name__)


class CommentEndpoint(BaseEndpoint):
    async def list(
        self,
        commentable_id: int = None,
        commentable_type: str = None,
        page: int = None,
        limit: int = None,
        desc: int = None,
    ) -> List[Comment] | RequestError:
        """
        List comments.

        :param commentable_id: must be a number.
        :param commentable_type: Must be one of: Topic, User.
        :param page: Must be a number between 1 and 100000.
        :param limit: 30 maximum.
        :param desc: Must be one of: 1, 0.
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/comments",
            params=filter_none_parameters(
                {
                    "commentable_id": commentable_id,
                    "commentable_type": commentable_type,
                    "page": page,
                    "limit": limit,
                    "desc": desc,
                }
            ),
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return [Comment.from_dict(com) for com in response]

        logger.debug(
            f"Bad Request(list): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def ById(self, id: int) -> Comment | RequestError:
        """
        Show a comment.

        :param id: must be a number.
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/comments/{id}",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return Comment.from_dict(response)

        logger.debug(
            f"Bad Request(ById): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def create(
        self,
        broadcast: bool = None,
        body: str = None,
        commentable_id: int = None,
        commentable_type: str = None,
        is_offtopic: bool = None,
        frontend: bool = None,
    ):
        """
        Create a comment.

        Requires comments oauth scope.

        :param broadcast: Must be one of: true, false.
        :param body: Must be a String.
        :param commentable_id: Must be a number.
        :param commentable_type: Must be one of: Topic, User, Anime, Manga, Character, Person, Article, Club, ClubPage, Collection, Critique, Review.  # NOQA
        :param is_offtopic: Must be one of: true, false.
        :param frontend: Must be one of: true, false.
        """

        response = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/api/comments",
            json=filter_none_parameters(
                {
                    "broadcast": broadcast,
                    "comment": filter_none_parameters(
                        {
                            "body": body,
                            "commentable_id": commentable_id,
                            "commentable_type": commentable_type,
                            "is_offtopic": is_offtopic,
                        }
                    ),
                    "frontend": frontend,
                }
            ),
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return Comment.from_dict(response)

        logger.debug(
            f"Bad Request(create): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def update(
        self, id: int, body: str, frontend: bool = None
    ) -> Comment | RequestError:
        """
        Update a comment. Requires comments oauth scope.

        Use abuse_requests to change is_offtopic field.

        :param body: string.
        :param frontend: string.
        """

        response = await self._request.make_request(
            "PATCH",
            url=f"{self._base_url}/api/comments/{id}",
            json=filter_none_parameters(
                {"comment": {"body": body}, "frontend": frontend}
            ),
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return Comment.from_dict(response)

        logger.debug(
            f"Bad Request(update): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def delete(self, id: int) -> str | RequestError:
        """
        Destroy a comment.
        Requires comments oauth scope.

        :param id: must be a number
        """
        response = await self._request.make_request(
            "DELETE",
            url=f"{self._base_url}/api/comments/{id}",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return response["notice"]

        logger.debug(
            f"Bad Request(delete): status - {response.status_code}: info - {str(response)}"
        )

        return response



import logging
from typing import List

from shikimori.types.calendar import Calendar
from .base import BaseEndpoint
from ..exceptions import RequestError
from ..utils.filter import filter_none_parameters

logger = logging.getLogger(__name__)


class CalendarEndpoint(BaseEndpoint):
    async def list(self, censored: bool = None) -> List[Calendar] | RequestError:
        """
        Show a calendar.

        :param censored: Set to false to allow hentai, yaoi and yuri
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/calendar",
            headers=self.headers,
            params=filter_none_parameters({"censored": censored}),
        )

        if not isinstance(response, RequestError):
            return [Calendar.from_dict(calendar) for calendar in response]

        logger.debug(
            f"Bad Request(list): status - {response.status_code}: info - {str(response)}"
        )

        return response



import dataclasses
from enum import Enum


class OrderEnum(Enum):
    ID = "id"
    ID_DESC = "id_desc"
    RANKED = "ranked"
    KIND = "kind"
    POPULARITY = "popularity"
    NAME = "name"
    AIRED_ON = "aired_on"
    EPISODES = "episodes"
    STATUS = "status"
    RANDOM = "random"
    RANKED_RANDOM = "ranked_random"
    RANKED_SHIKI = "ranked_shiki"
    CREATED_AT = "created_at"
    CREATED_AT_DESC = "created_at_desc"

    def __str__(self):
        return self.value


class GenreEntryTypeEnum(Enum):
    ANIME = "Anime"
    MANGA = "Manga"

    def __str__(self):
        return self.value


class UserRateTargetTypeEnum(Enum):
    ANIME = "Anime"
    MANGA = "Manga"

    def __str__(self):
        return self.value


class UserRateStatusEnum(Enum):
    PLANNED = "planned"
    WATCHING = "watching"
    REWATCHING = "rewatching"
    COMPLETED = "completed"
    ON_HOLD = "on_hold"
    DROPPED = "dropped"

    def __str__(self):
        return self.value


class UserRateOrderFieldEnum(Enum):
    ID = "id"
    UPDATED_AT = "updated_at"


class SortOrderEnum(Enum):
    ASC = "asc"
    DESC = "desc"


@dataclasses.dataclass
class UserRateOrderInputType:
    field: UserRateOrderFieldEnum
    order: SortOrderEnum



from dataclasses import dataclass
from shikimori.types.animes import Anime
from shikimori.utils.filter import handle_none_data


@dataclass
class Calendar:
    next_episode: int
    next_episode_at: str
    duration: int
    anime: Anime

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            next_episode=int(data.get("next_episode")),
            next_episode_at=data.get("next_episode_at"),
            duration=data.get("duration"),
            anime=Anime.from_dict(data.get("anime")),
        )



from dataclasses import dataclass
from shikimori.constants import SHIKIMORI_URL
from shikimori.utils.filter import handle_none_data


@dataclass
class ScreenShot:
    preview: str
    original: str

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(preview=data.get("preview"), original=data.get("original"))

    @property
    def original_url(self) -> str:
        return f"{SHIKIMORI_URL}{self.original}"

    @property
    def preview_url(self) -> str:
        return f"{SHIKIMORI_URL}{self.preview}"



import logging

from shikimori.types.episode_notification import EpisodeNotification
from .base import BaseEndpoint
from ..exceptions import RequestError
from ..utils.filter import filter_none_parameters

logger = logging.getLogger(__name__)


class EpisodeNotificationEndpoint(BaseEndpoint):
    async def notify(
        self,
        token: str,
        anime_id: int,
        episode: int,
        aired_at: str,
        is_fandub: bool = None,
        is_raw: bool = None,
        is_subtitles: bool = None,
        is_anime365: bool = None,
    ) -> EpisodeNotification | RequestError:
        """
        Notify shikimori about anime episode release.

        :param token: Private token required to access this api
        :param anime_id: Must be a number.
        :param episode: Must be a number.
        :param aired_at: Must be a date in iso8601 YYYY-MM-DDThh:mm:ss±hh format
        :param is_fandub: Must be one of: true, false, 1, 0.
        :param is_raw: Must be one of: true, false, 1, 0.
        :param is_subtitles: Must be one of: true, false, 1, 0.
        :param is_anime365: Must be one of: true, false, 1, 0.
        """
        response = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/api/v2/episode_notifications",
            json={
                "episode_notification": filter_none_parameters(
                    {
                        "anime_id": anime_id,
                        "episode": episode,
                        "aired_at": aired_at,
                        "is_fandub": is_fandub,
                        "is_raw": is_raw,
                        "is_subtitles": is_subtitles,
                        "is_anime365": is_anime365,
                    }
                ),
                "token": token,
            },
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return EpisodeNotification.from_dict(response)

        logger.debug(
            f"Bad Request(notify): status - {response.status_code}: info - {str(response)}"
        )

        return response



import logging
from typing import List

from shikimori.types.topics import Topic, Status, TopicReview
from .base import BaseEndpoint
from ..exceptions import RequestError
from ..utils.filter import filter_none_parameters

logger = logging.getLogger(__name__)


class TopicsEndpoint(BaseEndpoint):
    async def list(
        self,
        page: int = None,
        limit: int = None,
        forum: str = None,
        linked_id: int = None,
        linked_type: str = None,
        type: str = None,
    ) -> List[Topic] | RequestError:
        """
        List topics.

        :param page: Must be a number between 1 and 100000.
        :param limit: 30 maximum
        :param forum: Must be one of: all, animanga, site, games, vn, contests, offtopic, clubs, my_clubs, critiques, news, collections, articles, cosplay.
        :param linked_id: Used together with linked_type
        :param linked_type: Used together with linked_id. Must be one of: Anime, Manga, Ranobe, Character, Person, Club, ClubPage, Critique, Review, Contest, CosplayGallery, Collection, Article.
        :param type: Must be one of: Topic, Topics::ClubUserTopic, Topics::EntryTopic, Topics::EntryTopics::AnimeTopic, Topics::EntryTopics::ArticleTopic, Topics::EntryTopics::CharacterTopic, Topics::EntryTopics::ClubPageTopic, Topics::EntryTopics::ClubTopic, Topics::EntryTopics::CollectionTopic, Topics::EntryTopics::ContestTopic, Topics::EntryTopics::CosplayGalleryTopic, Topics::EntryTopics::MangaTopic, Topics::EntryTopics::PersonTopic, Topics::EntryTopics::RanobeTopic, Topics::EntryTopics::CritiqueTopic, Topics::EntryTopics::ReviewTopic, Topics::NewsTopic, Topics::NewsTopics::ContestStatusTopic.
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/topics",
            headers=self.headers,
            params=filter_none_parameters(
                {
                    "page": page,
                    "limit": limit,
                    "forum": forum,
                    "linked_id": linked_id,
                    "linked_type": linked_type,
                    "type": type,
                }
            ),
        )

        if not isinstance(response, RequestError):
            return [Topic.from_dict(topic) for topic in response]

        logger.debug(
            f"Bad Request(list): status - {response.status_code}: info - {str(response)}"
        )
        return response

    async def updates(
        self, page: int = None, limit: int = None
    ) -> List[Status] | RequestError:
        """
        NewsTopics about database updates.

        :param page: Must be a number between 1 and 100000.
        :param limit: 30 maximum
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/topics/updates",
            headers=self.headers,
            params=filter_none_parameters(
                {
                    "page": page,
                    "limit": limit,
                }
            ),
        )

        if not isinstance(response, RequestError):
            return [Status.from_dict(t) for t in response]

        logger.debug(
            f"Bad Request(updates): status - {response.status_code}: info - {str(response)}"
        )
        return response

    async def hot(self, limit: int = None) -> List[Topic] | RequestError:
        """
        Hot topics.

        :param limit: 10 maximum
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/topics/hot",
            headers=self.headers,
            params=filter_none_parameters(
                {
                    "limit": limit,
                }
            ),
        )

        if not isinstance(response, RequestError):
            return [Topic.from_dict(topic) for topic in response]

        logger.debug(
            f"Bad Request(hot): status - {response.status_code}: info - {str(response)}"
        )
        return response

    async def ById(self, id: int) -> Topic | RequestError:
        """
        Show a topic.

        :param id: must be a number
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/topics/{id}",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return TopicReview.from_dict(response)

        logger.debug(
            f"Bad Request(byId): status - {response.status_code}: info - {str(response)}"
        )
        return response

    async def create(
        self,
        body: str,
        forum_id: int,
        title: str,
        user_id: int,
        type: str = "Topic",
        linked_id: int = None,
        linked_type: str = None,
    ) -> Topic | RequestError:
        """
        Create a topic.

        Requires topics oauth scope.

        :param body: Must be a String.
        :param forum_id: Must be a number.
        :param title: Must be a String.
        :param user_id: Must be a number. .
        :param type: Must be one of: Topic.
        :param linked_id: Must be a number.
        :param linked_type: Must be one of: Anime, Manga, Ranobe, Character, Person, Club, ClubPage, Critique, Review, Contest, CosplayGallery, Collection, Article.
        """
        response = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/api/topics",
            headers=self.headers,
            json={
                "topic": filter_none_parameters(
                    {
                        "body": body,
                        "forum_id": forum_id,
                        "title": title,
                        "user_id": user_id,
                        "type": type,
                        "linked_id": linked_id,
                        "linked_type": linked_type,
                    }
                )
            },
        )

        if not isinstance(response, RequestError):
            return TopicReview.from_dict(response)

        logger.debug(
            f"Bad Request(create): status - {response.status_code}: info - {str(response)}"
        )
        return response

    async def update(
        self,
        id: int,
        body: str = None,
        linked_id: int = None,
        linked_type: str = None,
        title: str = None,
    ) -> Topic | RequestError:
        """
        Update a topic.

        Requires topics oauth scope.

        :param id: Must be a number.
        :param body: Must be a String.
        :param linked_id: Must be a number.
        :param linked_type: Must be one of: Anime, Manga, Ranobe, Character, Person, Club, ClubPage, Critique, Review, Contest, CosplayGallery, Collection, Article.
        :param title: Must be a String.
        """
        response = await self._request.make_request(
            "PATCH",
            url=f"{self._base_url}/api/topics/{id}",
            headers=self.headers,
            json={
                "topic": filter_none_parameters(
                    {
                        "body": body,
                        "title": title,
                        "linked_id": linked_id,
                        "linked_type": linked_type,
                    }
                )
            },
        )

        if not isinstance(response, RequestError):
            return TopicReview.from_dict(response)

        logger.debug(
            f"Bad Request(update): status - {response.status_code}: info - {str(response)}"
        )
        return response

    async def delete(self, id: int) -> None | RequestError:
        """
        Destroy a topic.

        Requires topics oauth scope.

        :param id: Must be a number
        """
        response = await self._request.make_request(
            "DELETE",
            url=f"{self._base_url}/api/topics/{id}",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return response["notice"]

        logger.debug(
            f"Bad Request(update): status - {response.status_code}: info - {str(response)}"
        )
        return response



import logging

from shikimori.types.style import Style
from .base import BaseEndpoint
from ..exceptions import RequestError

logger = logging.getLogger(__name__)


class StylesEndpoint(BaseEndpoint):
    async def ById(self, id: int) -> Style | RequestError:
        """
        Show a style.

        :param id: must be a number
        """
        response = await self._request.make_request(
            "GET", url=f"{self._base_url}/api/styles/{id}", headers=self.headers
        )

        if not isinstance(response, RequestError):
            return Style.from_dict(response)

        logger.debug(
            f"Bad Request(ById): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def preview(self, css: str) -> Style | RequestError:
        """
        Preview a style.

        :param css: Must be a String
        """
        response = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/api/styles/preview",
            json={"style": {"css": css}},
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return Style.from_dict(response)

        logger.debug(
            f"Bad Request(preview): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def create(
        self, css: str, name: str, owner_id: int, owner_type: str
    ) -> Style | RequestError:
        """
        Create a style.

        :param css: Must be a String
        :param name: Must be a String
        :param owner_id: Must be a number.
        :param owner_type: Must be one of: User, Club.
        """
        response = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/api/styles",
            json={
                "style": {
                    "css": css,
                    "name": name,
                    "owner_id": owner_id,
                    "owner_type": owner_type,
                }
            },
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return Style.from_dict(response)

        logger.debug(
            f"Bad Request(create): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def update(
        self, id: int, css: str = None, name: str = None
    ) -> Style | RequestError:
        """
        Update a style.

        :param id: Must be a number
        :param css: Must be a String
        :param name: Must be a String
        """
        response = await self._request.make_request(
            "PATCH",
            url=f"{self._base_url}/api/styles/{id}",
            json={
                "style": {
                    "css": css,
                    "name": name,
                }
            },
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return Style.from_dict(response)

        logger.debug(
            f"Bad Request(update): status - {response.status_code}: info - {str(response)}"
        )

        return response



import logging

from shikimori.types.abuse_requests import AbuseRequest
from .base import BaseEndpoint
from ..exceptions import RequestError
from ..utils.filter import filter_none_parameters

logger = logging.getLogger(__name__)


class AbuseRequestEndpoint(BaseEndpoint):
    async def AsOfftopic(self, comment_id: str | int) -> AbuseRequest | RequestError:
        """
        Mark comment as offtopic
        Request will be sent to moderators.

        :param comment_id: Must be a number.
        """

        response = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/api/v2/abuse_requests/offtopic",
            headers=self.headers,
            json={"comment_id": comment_id},
        )

        if not isinstance(response, RequestError):
            return AbuseRequest.from_dict(response)

        logger.debug(
            f"Bad Request(mark_comment_as_offtopic): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def AsReview(
        self, topic_id: str | int = None, comment_id: str | int = None
    ) -> None | RequestError:
        """
        Convert comment to review.

        Request will be sent to moderators.

        :param topic_id: Must be a number.
        :param comment_id: Must be a number.
        """

        response = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/api/v2/topics/{topic_id}/ignore",
            headers=self.headers,
            json=filter_none_parameters(
                {"topic_id": topic_id, "comment_id": comment_id}
            ),
        )

        if not isinstance(response, RequestError):
            return None

        logger.debug(
            f"Bad Request(convert_comment_to_review): status - {response.status_code}: info - {str(response)}"
        )
        return response

    async def CreateViolation(
        self,
        topic_id: str | int = None,
        comment_id: str | int = None,
        reason: str = None,
    ) -> None | RequestError:
        """
        Create abuse about violation of site rules.

        Request will be sent to moderators.

        :param comment_id: Must be a number.
        :param topic_id: Must be a number.
        :param reason: Must be a string.
        """
        response = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/api/v2/abuse_requests/abuse",
            headers=self.headers,
            json=filter_none_parameters(
                {"topic_id": topic_id, "comment_id": comment_id, "reason": reason}
            ),
        )

        if not isinstance(response, RequestError):
            return None

        logger.debug(
            f"Bad Request(create_abuse_about_violation_of_site_rules): status - {response.status_code}: info - {str(response)}"  # NOQA
        )

        return response

    async def SpoilerInContent(
        self,
        topic_id: str | int = None,
        comment_id: str | int = None,
        reason: str = None,
    ) -> None | RequestError:
        """
        Create abuse about spoiler in content
        Request will be sent to moderators.

        :param comment_id: Must be a number.
        :param topic_id: Must be a number.
        :param reason: Must be a string.
        """
        response = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/api/v2/abuse_requests/spoiler",
            headers=self.headers,
            json=filter_none_parameters(
                {"topic_id": topic_id, "comment_id": comment_id, "reason": reason}
            ),
        )

        if not isinstance(response, RequestError):
            return None

        logger.debug(
            f"Bad Request(create_abuse_about_spoiler_in_content): status - {response.status_code}: info - {str(response)}"  # NOQA
        )

        return response



from abc import ABC

from shikimori.base import BaseLimiter


class BaseEndpoint(ABC):
    def __init__(self, base_url: str, request: BaseLimiter, user_agent: str):
        self._base_url = base_url
        self._request = request
        self._user_agent = user_agent

    @property
    def headers(self) -> dict:
        return {"User-Agent": self._user_agent}



from dataclasses import dataclass
from shikimori.types.user import User
from shikimori.utils.filter import handle_none_data
from shikimori.types.comment import CommentBrief


@dataclass
class Ban:
    id: int
    user_id: int
    comment: CommentBrief
    moderator_id: int
    reason: str
    created_at: str
    duration_minutes: int
    user: User
    moderator: User

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            id=data.get("id"),
            user_id=data.get("user_id"),
            comment=CommentBrief.from_dict(data.get("comment")),
            moderator_id=data.get("moderator_id"),
            reason=data.get("reason"),
            created_at=data.get("created_at"),
            duration_minutes=data.get("duration_minutes"),
            user=User.from_dict(data.get("user")),
            moderator=User.from_dict(data.get("moderator")),
        )



from abc import ABC, abstractmethod
from .exceptions import RequestError
from typing import Any

__all__ = ["BaseRequest", "BaseLimiter"]


class BaseRequest(ABC):
    """
    Base class for request
    """

    @abstractmethod
    async def make_request(
        self, method: str, headers: dict = None, **kwargs
    ) -> RequestError | Any:
        """
        :param method: one of [get, post, patch, delete]
        :param headers: headers of request
        :param kwargs: other parameters
        """
        raise NotImplementedError


class BaseLimiter(ABC):
    """
    Interface for request limiter
    """

    @abstractmethod
    async def make_request(self, method: str, **kwargs) -> RequestError | Any:
        """
        :param method: one of [get, post, patch, delete]
        :param kwargs: other parameters
        """
        raise NotImplementedError



import logging

from shikimori.types.topics import Forum
from .base import BaseEndpoint
from ..exceptions import RequestError

logger = logging.getLogger(__name__)


class ForumEndpoint(BaseEndpoint):
    async def list(self) -> RequestError | list[Forum]:
        """List of forums"""
        response = await self._request.make_request(
            "GET", url=f"{self._base_url}/api/forums", headers=self.headers
        )

        if not isinstance(response, RequestError):
            return [Forum.from_dict(s) for s in response]

        logger.debug(
            f"Bad Request(add): status - {response.status_code}: info - {str(response)}"
        )

        return response



from dataclasses import dataclass

from shikimori.utils.filter import handle_none_data


@dataclass
class EpisodeNotification:
    id: int
    anime_id: int
    episode: int
    is_raw: bool
    is_subtitles: bool
    is_fandub: bool
    is_anime365: bool
    topic_id: bool

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            id=data.get("id"),
            anime_id=data.get("anime_id"),
            episode=data.get("episode"),
            is_raw=data.get("is_raw"),
            is_subtitles=data.get("is_subtitles"),
            is_fandub=data.get("is_fandub"),
            is_anime365=data.get("is_anime365"),
            topic_id=data.get("topic_id"),
        )



from dataclasses import dataclass


@dataclass
class AuthOptions:
    client_id: str
    redirect_uri: str
    client_secret: str
    scopes: list[str] | None = None


@dataclass
class AccessTokenData:
    access_token: str
    token_type: str
    expires_in: int
    refresh_token: str
    scope: str
    created_at: int



from dataclasses import dataclass
from shikimori.types.animes import Anime
from shikimori.types.character import Character
from shikimori.types.manga import Manga
from .photo import Photo
from ..utils.filter import handle_none_data


@dataclass
class Date:
    day: int = None
    month: int = None
    year: int = None

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(day=data.get("day"), month=data.get("month"), year=data.get("year"))


@dataclass
class Works:
    anime: Anime | None
    manga: Manga | None
    role: str

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            anime=Anime.from_dict(data.get("anime")),
            manga=Manga.from_dict(data.get("manga")),
            role=data.get("role"),
        )


@dataclass
class Role:
    animes: list[Anime]
    characters: list[Character]

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            animes=[
                Anime.from_dict(anime_data) for anime_data in data.get("animes", [])
            ],
            characters=[
                Character.from_dict(char_data)
                for char_data in data.get("characters", [])
            ],
        )


@dataclass
class People:
    id: int
    name: str
    russian: str
    image: Photo
    url: str
    japanese: str
    job_title: str
    birth_on: Date
    deceased_on: Date
    website: str
    groupped_roles: list[str | int]
    works: list[Works]
    roles: list[Role]
    topic_id: int
    person_favoured: bool
    producer: bool
    producer_favoured: bool
    mangaka: bool
    mangaka_favoured: bool
    seyu: bool
    seyu_favoured: bool
    updated_at: str
    thread_id: int
    birthday: Date

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            id=data.get("id"),
            name=data.get("name"),
            russian=data.get("russian"),
            image=Photo.from_dict(data.get("image")),
            url=data.get("url"),
            japanese=data.get("japanese"),
            job_title=data.get("job_title"),
            birth_on=Date.from_dict(data.get("birth_on")),
            deceased_on=Date.from_dict(data.get("deceased_on")),
            website=data.get("website"),
            groupped_roles=data.get("groupped_roles", []),
            works=[Works.from_dict(works_data) for works_data in data.get("works", [])],
            roles=[Role.from_dict(role_data) for role_data in data.get("roles", [])],
            topic_id=data.get("topic_id"),
            person_favoured=data.get("person_favoured"),
            producer=data.get("producer"),
            producer_favoured=data.get("producer_favoured"),
            mangaka=data.get("mangaka"),
            mangaka_favoured=data.get("mangaka_favoured"),
            seyu=data.get("seyu"),
            seyu_favoured=data.get("seyu_favoured"),
            updated_at=data.get("updated_at"),
            thread_id=data.get("thread_id"),
            birthday=Date.from_dict(data.get("birthday")),
        )



from dataclasses import dataclass

from shikimori.utils.filter import handle_none_data


@dataclass
class Review:
    id: int
    user_id: int
    anime_id: int
    manga_id: int
    body: str
    opinion: str
    is_written_before_release: bool
    created_at: str
    updated_at: str
    comments_count: int
    cached_votes_up: int
    cached_votes_down: int
    changed_at: str

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            id=data.get("id"),
            user_id=data.get("user_id"),
            anime_id=data.get("anime_id"),
            manga_id=data.get("manga_id"),
            body=data.get("body"),
            opinion=data.get("opinion"),
            is_written_before_release=data.get("is_written_before_release"),
            created_at=data.get("created_at"),
            updated_at=data.get("updated_at"),
            comments_count=data.get("comments_count"),
            cached_votes_up=data.get("cached_votes_up"),
            cached_votes_down=data.get("cached_votes_down"),
            changed_at=data.get("changed_at"),
        )



import logging

from .base import BaseEndpoint
from ..exceptions import RequestError
from ..utils.filter import filter_none_parameters

logger = logging.getLogger(__name__)


class FavoritesEndpoint(BaseEndpoint):
    async def add(
        self, id: int, linked_type: str, kind: str = None
    ) -> str | RequestError:
        """
        Create a favorite.

        :param id: must be a number
        :param linked_type: Must be one of: Anime, Manga, Ranobe, Person, Character
        :param kind: Must be one of: common, seyu, mangaka, producer, person
        """
        url = f"{self._base_url}/api/favorites/{linked_type}/{id}"

        if kind:
            url += f"/{kind}"

        response = await self._request.make_request(
            "POST", url=url, headers=self.headers
        )

        if not isinstance(response, RequestError):
            return response["notice"]

        logger.debug(
            f"Bad Request(add): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def delete(self, id: int, linked_type: str) -> str | RequestError:
        """
        Destroy a favorite.

        :param id: must be a number
        :param linked_type: Must be one of: Anime, Manga, Ranobe, Person, Character
        """
        url = f"{self._base_url}/api/favorites/{linked_type}/{id}"

        response = await self._request.make_request(
            "DELETE", url=url, headers=self.headers
        )

        if not isinstance(response, RequestError):
            return response["notice"]

        logger.debug(
            f"Bad Request(delete): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def reorder(self, id: int, new_index: int = None) -> None | RequestError:
        """
        Assign a new position to a favorite.

        :param id: must be a number.
        :param new_index: must be a number.
        """
        response = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/api/favorites/{id}",
            headers=self.headers,
            json=filter_none_parameters({"new_index": new_index}),
        )

        if not isinstance(response, RequestError):
            return

        logger.debug(
            f"Bad Request(reorder): status - {response.status_code}: info - {str(response)}"
        )

        return response



from dataclasses import dataclass

from ..utils.filter import handle_none_data


@dataclass
class ExternalLink:
    id: int
    kind: str
    url: str
    source: str
    entry_id: int
    entry_type: str
    created_at: str
    updated_at: str
    imported_at: str

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            id=data.get("id"),
            kind=data.get("kind"),
            url=data.get("url"),
            source=data.get("source"),
            entry_id=data.get("entry_id"),
            entry_type=data.get("entry_type"),
            created_at=data.get("created_at"),
            updated_at=data.get("updated_at"),
            imported_at=data.get("imported_at"),
        )



import logging

from shikimori.types.constants import (
    SmileConstant,
    ClubConstant,
    AnimeConstant,
    MangaConstant,
    UserRateConstant,
)
from .base import BaseEndpoint
from ..exceptions import RequestError

logger = logging.getLogger(__name__)


class ConstantsEndpoint(BaseEndpoint):
    async def animes(self) -> AnimeConstant | RequestError:
        response = await self._request.make_request(
            "GET",
            headers=self.headers,
            url=f"{self._base_url}/api/constants/anime",
        )

        if not isinstance(response, RequestError):
            return AnimeConstant.from_dict(response)

        logger.debug(
            f"Bad Request(animes): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def mangas(self) -> MangaConstant | RequestError:
        response = await self._request.make_request(
            "GET",
            headers=self.headers,
            url=f"{self._base_url}/api/constants/manga",
        )

        if not isinstance(response, RequestError):
            return MangaConstant.from_dict(response)

        logger.debug(
            f"Bad Request(manga): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def user_rates(self) -> UserRateConstant | RequestError:
        response = await self._request.make_request(
            "GET",
            headers=self.headers,
            url=f"{self._base_url}/api/constants/manga",
        )

        if not isinstance(response, RequestError):
            return UserRateConstant.from_dict(response)

        logger.debug(
            f"Bad Request(user_rate): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def clubs(self) -> ClubConstant | RequestError:
        response = await self._request.make_request(
            "GET",
            headers=self.headers,
            url=f"{self._base_url}/api/constants/manga",
        )

        if not isinstance(response, RequestError):
            return ClubConstant.from_dict(response)

        logger.debug(
            f"Bad Request(club): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def smileys(self) -> list[SmileConstant] | RequestError:
        response = await self._request.make_request(
            "GET",
            headers=self.headers,
            url=f"{self._base_url}/api/constants/smileys",
        )

        if not isinstance(response, RequestError):
            return [SmileConstant.from_dict(s) for s in response]

        logger.debug(
            f"Bad Request(smileys): status - {response.status_code}: info - {str(response)}"
        )

        return response



import logging
import aiohttp
from .base import BaseRequest
from .exceptions import RequestError
from typing import Any

logger = logging.getLogger(__name__)

__all__ = ["Request"]


class Request(BaseRequest):
    """
    A class for sending HTTP requests.

    This class extends the functionality of the BaseRequest class
    to provide methods for sending various types of HTTP requests.
    """

    def __init__(self, token: str = None):
        self._token = token

    def set_token(self, token: str) -> None:
        self._token = token

    async def _request(
        self, method: str, headers: dict = None, **kwargs
    ) -> Any | RequestError:
        """
        Sends an HTTP request with the specified method.

        :param method: The HTTP method (e.g., 'GET', 'POST', 'DELETE').
        :param headers: A dictionary of HTTP headers to be included in the request. Defaults to None.
        :param **kwargs: Additional keyword arguments to be passed to the underlying request function.

        Returns:
            Union[Any, RequestError]: The response object corresponding to the HTTP request, or a RequestError if an error occurs.

        """
        try:
            if self._token:
                headers["Authorization"] = f"Bearer {self._token}"

            async with aiohttp.ClientSession(headers=headers) as session:
                async with session.request(method, **kwargs) as response:
                    response.raise_for_status()
                    return await response.json()

        except aiohttp.ClientResponseError as exc:
            logger.debug(f"Error occurred with {method} request - {exc}")
            return RequestError(exc.message, exc.status)

    async def make_request(self, method: str, **kwargs):
        methods = ["GET", "POST", "PATCH", "DELETE"]

        if method not in methods:
            raise ValueError(f"Method {method} is not allowed")

        return await self._request(method, **kwargs)



import logging
from typing import List

from shikimori.types.bans import Ban
from shikimori.types.club import Club
from shikimori.types.message import MessageInfo
from shikimori.types.user import (
    User,
    UserInfo,
    UserInfoInc,
    Rate,
    Favourites,
    UnreadMessages,
    HistoryObj,
)
from .base import BaseEndpoint
from ..exceptions import RequestError
from ..utils.filter import filter_none_parameters

logger = logging.getLogger(__name__)


class UserEndpoint(BaseEndpoint):
    async def list(
        self, page: int = None, limit: int = None, search: str = None
    ) -> List[User] | RequestError:
        """
        list users.

        :param page: Must be a number between 1 and 100000.
        :param limit: 100 maximum
        :param search: Must be a String
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/users",
            params=filter_none_parameters(
                {"search": search, "page": page, "limit": limit}
            ),
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return [User.from_dict(u) for u in response]

        logger.debug(
            f"Bad Request(list): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def ById(self, id: int, is_nickname: str = None) -> UserInfo | RequestError:
        """
        Show a user.

        :param id: must be a number
        :param is_nickname: 1 if you want to get user by its nickname
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/users/{id}",
            params=filter_none_parameters({"is_nickname": is_nickname}),
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return UserInfo.from_dict(response)

        logger.debug(
            f"Bad Request(ById): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def info(self, id: int) -> UserInfoInc | RequestError:
        """
        Show user's brief info.

        :param id: must be a number
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/users/{id}/info",
            headers=self.headers,
        )
        if not isinstance(response, RequestError):
            return UserInfoInc.from_dict(response)

        logger.debug(
            f"Bad Request(info): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def whoami(self) -> UserInfoInc | RequestError:
        """
        Show current user's brief info
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/users/whoami",
            headers=self.headers,
        )
        if not isinstance(response, RequestError):
            return UserInfoInc.from_dict(response)

        logger.debug(
            f"Bad Request(whoami): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def signOut(self) -> None | RequestError:
        """
        Sign out the user
        """
        response = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/api/users/sign_out",
            headers=self.headers,
        )
        if not isinstance(response, RequestError):
            return

        logger.debug(
            f"Bad Request(signOut): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def friends(
        self, page: int = None, limit: int = None
    ) -> List[User] | RequestError:
        """
        Show user's friends.

        :param limit: 100 maximum.
        :param page: Must be a number between 1 and 100000.
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/users/friends",
            headers=self.headers,
            json=filter_none_parameters({"page": page, "limit": limit}),
        )
        if not isinstance(response, RequestError):
            return [User.from_dict(u) for u in response]

        logger.debug(
            f"Bad Request(friends): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def clubs(self, id: int) -> List[Club] | RequestError:
        """
        Show user's clubs.

        :param id: must be a number
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/users/{id}/clubs",
            headers=self.headers,
        )
        if not isinstance(response, RequestError):
            return [Club.from_dict(club) for club in response]

        logger.debug(
            f"Bad Request(clubs): status - {response.status_code}: info - {str(response)}"
        )
        return response

    async def animeRates(
        self,
        id: int,
        limit: int = None,
        page: int = None,
        status: str = None,
        censored: bool = None,
    ) -> List[Rate] | RequestError:
        """
        Show user's anime list.

        :param id: must be a number
        :param limit: Must be a number between 1 and 100000
        :param page: 5000 maximum
        :param status: Must be one of: planned, watching, rewatching, completed, on_hold, dropped.
        :param censored: Must be one of: true, false.
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/users/{id}/anime_rates",
            params=filter_none_parameters(
                {"limit": limit, "page": page, "status": status, "censored": censored}
            ),
            headers=self.headers,
        )
        if not isinstance(response, RequestError):
            return [Rate.from_dict(rate) for rate in response]

        logger.debug(
            f"Bad Request(animeRates): status - {response.status_code}: info - {str(response)}"
        )
        return response

    async def mangaRates(
        self,
        id: int,
        limit: int = None,
        page: int = None,
        censored: bool = None,
    ) -> List[Rate] | RequestError:
        """
        Show user's manga list.

        :param id: must be a number
        :param limit: Must be a number between 1 and 100000
        :param page: 5000 maximum
        :param censored: Must be one of: true, false.
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/users/{id}/manga_rates",
            params=filter_none_parameters(
                {"limit": limit, "page": page, "censored": censored}
            ),
            headers=self.headers,
        )
        if not isinstance(response, RequestError):
            return [Rate.from_dict(rate) for rate in response]

        logger.debug(
            f"Bad Request(MangaRates): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def favourites(self, id: int) -> Favourites | RequestError:
        """
        Show user's favourites.

        :param id: must be a number
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/users/{id}/favourites",
            headers=self.headers,
        )
        if not isinstance(response, RequestError):
            return Favourites.from_dict(response)

        logger.debug(
            f"Bad Request(favourites): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def messages(
        self, id: int, page: int = None, limit: int = None, type: str = None
    ) -> List[MessageInfo] | RequestError:
        """
        Show current user's messages.

        :param id: must be a number.
        :param page: Must be a number between 1 and 100000.
        :param limit: 100 maximum
        :param type: Must be one of: inbox, private, sent, news, notifications.
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/users/{id}/messages",
            headers=self.headers,
            params=filter_none_parameters({"page": page, "limit": limit, "type": type}),
        )

        if not isinstance(response, RequestError):
            return [MessageInfo.from_dict(msg) for msg in response]

        logger.debug(
            f"Bad Request(messages): status - {response.status_code}: info - {str(response)}"
        )
        return response

    async def unread(self, id: int) -> UnreadMessages | RequestError:
        """
        Show current user's unread messages counts.

        :param id: must be a number.
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/users/{id}/unread_messages",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return UnreadMessages.from_dict(response)

        logger.debug(
            f"Bad Request(unread): status - {response.status_code}: info - {str(response)}"
        )
        return response

    async def history(
        self,
        id: int,
        page: int = None,
        limit: int = None,
        target_type: str = None,
        target_id: int = None,
    ) -> List[HistoryObj] | RequestError:
        """
        Show user history.

        :param id: Must be a number.
        :param page: Must be a number between 1 and 100000.
        :param limit: 100 maximum
        :param target_type: Must be one of: Anime, Manga.
        :param target_id: Must be a number.
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/users/{id}/history",
            headers=self.headers,
            params=filter_none_parameters(
                {
                    "page": page,
                    "limit": limit,
                    "target_type": target_type,
                    "target_id": target_id,
                }
            ),
        )

        if not isinstance(response, RequestError):
            return [HistoryObj.from_dict(obj) for obj in response]

        logger.debug(
            f"Bad Request(history): status - {response.status_code}: info - {str(response)}"
        )
        return response

    async def bans(self, id: int) -> List[Ban] | RequestError:
        """
        Show user's bans.

        :param id: Must be a number.
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/users/{id}/bans",
            headers=self.headers,
        )
        if not isinstance(response, RequestError):
            return [Ban.from_dict(ban) for ban in response]

        logger.debug(
            f"Bad Request(bans): status - {response.status_code}: info - {str(response)}"
        )

        return response



from dataclasses import dataclass
from shikimori.types.user import User
from shikimori.utils.filter import handle_none_data


@dataclass
class Comment:
    id: int
    user_id: int
    commentable_id: int
    commentable_type: str
    body: str
    html_body: str
    created_at: str
    updated_at: str
    is_offtopic: bool
    is_summary: bool
    can_be_edited: bool
    user: User

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            id=data.get("id"),
            user_id=data.get("user_id"),
            commentable_id=data.get("commentable_id"),
            commentable_type=data.get("commentable_type"),
            body=data.get("body"),
            html_body=data.get("html_body"),
            created_at=data.get("created_at"),
            updated_at=data.get("updated_at"),
            is_offtopic=data.get("is_offtopic"),
            is_summary=data.get("is_summary"),
            can_be_edited=data.get("can_be_edited"),
            user=User.from_dict(data.get("user")),
        )


@dataclass
class CommentBrief:
    id: int
    commentable_id: int
    commentable_type: str
    body: str
    user_id: int
    created_at: str
    updated_at: str
    is_offtopic: bool

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            id=int(data.get("id")),
            commentable_id=data.get("commentable_id"),
            commentable_type=data.get("commentable_type"),
            body=data.get("body"),
            user_id=data.get("user_id"),
            created_at=data.get("created_at"),
            updated_at=data.get("updated_at"),
            is_offtopic=data.get("is_offtopic"),
        )



import logging
from shikimori.types.auth import AuthOptions, AccessTokenData

from shikimori.base import BaseLimiter
from shikimori.exceptions import RequestError

__all__ = ["Auth"]

logger = logging.getLogger(__name__)


class Auth:
    """
    Class for handling authentication with the Shikimori API.

    This class provides methods for obtaining access tokens and refreshing tokens.
    """

    def __init__(
        self,
        request: BaseLimiter,
        user_agent: str = None,
        options: AuthOptions | None = None,
        base_url: str = None,
    ):
        """
        Initialize the Auth class.

        :param request: An instance of the BaseLimiter class for making requests.
        :param user_agent:  User-agent string to be used in API requests. Defaults to None.
        :param options:  An instance of the AuthOptions class containing authentication options. Defaults to None.
        :param base_url:  Base URL for the Shikimori API. Defaults to None.
        """
        self._base_url = base_url
        self._headers = {"User-Agent": user_agent}
        self._request = request
        self._options = options

    async def get_access_token(self, auth_code: str) -> AccessTokenData | RequestError:
        """
        Obtain an access token using an authorization code.

        :param auth_code: Authorization code obtained during the OAuth authorization process.
        """
        body = {
            "grant_type": "authorization_code",
            "client_id": self._options.client_id,
            "client_secret": self._options.client_secret,
            "code": auth_code,
            "redirect_uri": self._options.redirect_uri,
        }

        resp = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/oauth/token",
            json=body,
            headers=self._headers,
        )

        if not isinstance(resp, RequestError):
            return AccessTokenData(**resp)

        logger.debug(
            f"Bad Request(refresh): status - {resp.status_code}: info - {str(resp)}"
        )

        return resp

    async def refresh(self, refresh_token: str) -> AccessTokenData | RequestError:
        """
        Refresh an access token using a refresh token.

        :param refresh_token: Refresh token obtained during the OAuth authorization process.
        """
        body = {
            "grant_type": "refresh_token",
            "client_id": self._options.client_id,
            "client_secret": self._options.client_secret,
            "refresh_token": refresh_token,
        }
        resp = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/oauth/token",
            json=body,
            headers=self._headers,
        )

        if not isinstance(resp, RequestError):
            return AccessTokenData(**resp)

        logger.debug(
            f"Bad Request(refresh): status - {resp.status_code}: info - {str(resp)}"
        )

        return resp

    @property
    def auth_url(self) -> str:
        """
        Generates a URL for obtaining an authorization code to authenticate requests.
        """
        return f"https://shikimori.one/oauth/authorize?client_id={self._options.client_id}&redirect_uri={self._options.redirect_uri}&response_type=code&scope={'+'.join(self._options.scopes) if self._options.scopes else ''}"  # NOQA



import logging

from shikimori.types.achievements import Achievement
from .base import BaseEndpoint
from ..exceptions import RequestError

logger = logging.getLogger(__name__)


class AchievementsEndpoint(BaseEndpoint):
    async def list(self, user_id: int) -> list[Achievement] | RequestError:
        """
        List user achievements.

        :param user_id: Must be a number.
        """
        response = await self._request.make_request(
            "GET",
            params={"user_id": user_id},
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return [Achievement.from_dict(obj) for obj in response]

        logger.debug(
            f"Bad Request(list): status - {response.status_code}: info - {str(response)}"
        )
        return response



import logging

from shikimori.types.people import People
from .base import BaseEndpoint
from ..exceptions import RequestError
from shikimori.types.character import CharacterBrief
from ..utils.filter import filter_none_parameters

logger = logging.getLogger(__name__)


class PeopleEndpoint(BaseEndpoint):
    async def ById(self, id: int) -> People | RequestError:
        """
        Show a person.

        :param id: must be a number
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/peoples/{id}",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return People.from_dict(response)

        logger.debug(
            f"Bad Request(ById): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def search(
        self, id: int, type: str = None
    ) -> list[CharacterBrief] | RequestError:
        """
        Search people.

        :param id: must be a number.
        :param type: Must be one of: seyu, mangaka, producer.
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/peoples/{id}",
            params=filter_none_parameters({"type": type}),
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return [CharacterBrief.from_dict(ch) for ch in response]

        logger.debug(
            f"Bad Request(search): status - {response.status_code}: info - {str(response)}"
        )

        return response



from shikimori.types.genres import Genre
from shikimori.types.photo import Photo
from dataclasses import dataclass
from shikimori.types.user_rates import UserRate
from ..utils.filter import handle_none_data


@dataclass
class Manga:
    id: int
    name: str
    russian: str
    image: Photo
    url: str
    kind: str
    score: str
    status: str
    aired_on: str
    released_on: str
    volumes: int
    chapters: int

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            volumes=data.get("volumes"),
            chapters=data.get("chapters"),
            id=data.get("id"),
            name=data.get("name"),
            russian=data.get("russian"),
            image=Photo.from_dict(data.get("image")),
            url=data.get("url"),
            kind=data.get("kind"),
            score=data.get("score"),
            status=data.get("status"),
            aired_on=data.get("aired_on"),
            released_on=data.get("released_on"),
        )


@dataclass
class MangaInfo(Manga):
    english: list[str]
    japanese: list[str]
    synonyms: list[str]
    license_name_ru: str
    description: str
    description_html: str
    description_source: str | None
    franchise: str
    favoured: bool
    anons: bool
    ongoing: bool
    thread_id: int
    topic_id: int
    myanimelist_id: int
    rates_scores_stats: list[dict]
    rates_statuses_stats: list[dict]
    licensors: list[str]
    genres: list[Genre]
    user_rate: UserRate | None

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            english=data.get("english", []),
            japanese=data.get("japanese", []),
            synonyms=data.get("synonyms", []),
            license_name_ru=data.get("license_name_ru"),
            description=data.get("description"),
            description_html=data.get("description_html"),
            description_source=data.get("description_source"),
            franchise=data.get("franchise"),
            favoured=data.get("favoured"),
            anons=data.get("anons"),
            ongoing=data.get("ongoing"),
            thread_id=data.get("thread_id"),
            topic_id=data.get("topic_id"),
            myanimelist_id=data.get("myanimelist_id"),
            rates_scores_stats=data.get("rates_scores_stats", []),
            rates_statuses_stats=data.get("rates_statuses_stats", []),
            licensors=data.get("licensors", []),
            genres=[
                Genre.from_dict(genre_data) for genre_data in data.get("genres", [])
            ],
            user_rate=(
                UserRate.from_dict(data.get("user_rate"))
                if data.get("user_rate")
                else None
            ),
            volumes=data.get("volumes"),
            chapters=data.get("chapters"),
            id=data.get("id"),
            name=data.get("name"),
            russian=data.get("russian"),
            image=Photo.from_dict(data.get("image")),
            url=data.get("url"),
            kind=data.get("kind"),
            score=data.get("score"),
            status=data.get("status"),
            aired_on=data.get("aired_on"),
            released_on=data.get("released_on"),
        )



import logging
from typing import List

from shikimori.types.animes import (
    Anime,
    AnimeInfo,
    Relation,
)
from shikimori.types.externalLink import ExternalLink
from shikimori.types.franchise import Franchise
from shikimori.types.roles import Role
from shikimori.types.screenshots import ScreenShot
from shikimori.types.topics import Topic
from .base import BaseEndpoint
from ..exceptions import RequestError
from ..utils.filter import filter_none_parameters

logger = logging.getLogger(__name__)


class AnimeEndpoint(BaseEndpoint):
    async def list(
        self,
        page: int = None,
        limit: int = None,
        order: str = None,
        kind: str = None,
        status: str = None,
        season: str = None,
        score: str = None,
        duration: str = None,
        rating: str = None,
        genre: str = None,
        studio: str = None,
        franchise: str = None,
        censored: str = None,
        mylist: str = None,
        ids: str = None,
        exclude_ids: str = None,
        search: str = None,
        genre_v2: str = None,
    ) -> list[Anime] | RequestError:
        """
        List animes.

        :param page: must be a number between 1 and 100000.
        :param limit: Must be a number, 50 - maximum.
        :param order: Must be one of: id, id_desc, ranked, kind, popularity, name, aired_on, episodes, status, random, ranked_random, ranked_shiki, created_at, created_at_desc.
        :param kind: Must be one of: tv, movie, ova, ona, special, tv_special, music, pv, cm, tv_13, tv_24, tv_48.
        :param status: Must be one of: anons, ongoing, released.
        :param season: ex - summer_2017, 2016, 2014_2016, 199x.
        :param score: Must be a number.
        :param duration: Must be one of: S - less than 10, D - less than 30, F - more than 30.
        :param rating: Must be one of: none, g, pg, pg_13, r, r_plus, rx.
        :param genre: List of genre ids separated by comma.
        :param studio: List of studio ids separated by comma.
        :param franchise: List of franchises separated by comma.
        :param censored: Must be one of: true, false.
        :param mylist: Must be one of: planned, watching, rewatching, completed, on_hold, dropped.
        :param ids: List of anime ids separated by comma.
        :param exclude_ids: List of anime ids separated by comma.
        :param search: Search phrase to filter animes by name.
        :param genre_v2: List of genre v2 ids separated by comma.
        """

        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/animes",
            params=filter_none_parameters(
                {
                    "page": page,
                    "limit": limit,
                    "order": order,
                    "kind": kind,
                    "status": status,
                    "season": season,
                    "studio": studio,
                    "score": score,
                    "duration": duration,
                    "rating": rating,
                    "genre": genre,
                    "franchise": franchise,
                    "censored": censored,
                    "mylist": mylist,
                    "ids": ids,
                    "exclude_ids": exclude_ids,
                    "search": search,
                    "genre_v2": genre_v2,
                }
            ),
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return [Anime.from_dict(anime) for anime in response]

        logger.debug(
            f"Bad Request(anime_list): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def ById(self, id: int) -> AnimeInfo | RequestError:
        """
        Show an anime.

        :param id: Must be a Number.
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/animes/{id}",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return AnimeInfo.from_dict(response)

        logger.debug(
            f"Bad Request(byId): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def roles(self, id: int) -> List[Role] | RequestError:
        """
        List anime roles.

        :param id: must be a number.
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/animes/{id}/roles",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return [Role.from_dict(role) for role in response]

        logger.debug(
            f"Bad Request(roles): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def similar(self, id: int) -> List[Anime] | RequestError:
        """
        List similar animes.

        :param id: must be a number.
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/animes/{id}/similar",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return [Anime.from_dict(anime) for anime in response]

        logger.debug(
            f"Bad Request(similar): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def related(self, id: int) -> List[Relation] | RequestError:
        """
        List related animes.

        :param id: must be a number.
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/animes/{id}/related",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return [Relation.from_dict(relation) for relation in response]

        logger.debug(
            f"Bad Request(related): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def screenshots(self, id: int) -> List[ScreenShot] | RequestError:
        """
        List screenshots anime.

        :param id: must be a number.
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/animes/{id}/screenshots",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return [ScreenShot.from_dict(s) for s in response]

        logger.debug(
            f"Bad Request(related): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def franchise(self, id: int) -> Franchise | RequestError:
        """
        Show the whole franchise.

        :param id: must be a number.
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/animes/{id}/franchise",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return Franchise.from_dict(response)

        logger.debug(
            f"Bad Request(franchise): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def externalLinks(self, id: int) -> List[ExternalLink] | RequestError:
        """
        Show the externalLinks of an anime.

        :param id: must be a number.
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/animes/{id}/external_links",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return [ExternalLink.from_dict(s) for s in response]

        logger.debug(
            f"Bad Request(externalLinks): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def topics(
        self,
        id: int,
        page: int = None,
        limit: int = None,
        kind: str = None,
        episode: int = None,
    ) -> List[Topic] | RequestError:
        """
        list of topics that relate to anime.

        :param id: Must be a number.
        :param page: Must be a number between 1 and 100000.
        :param limit: 30 maximum.
        :param kind: Must be one of: anons, ongoing, released, episode.
        :param episode: Must be a number.
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/animes/{id}/topics",
            headers=self.headers,
            params=filter_none_parameters(
                {"page": page, "limit": limit, "kind": kind, "episode": episode}
            ),
        )

        if not isinstance(response, RequestError):
            return [Topic.from_dict(topic) for topic in response]

        return response



import logging

from shikimori.types.genres import Genre
from .base import BaseEndpoint
from ..exceptions import RequestError

logger = logging.getLogger(__name__)


class GenreEndpoint(BaseEndpoint):
    async def list(self) -> list[Genre] | RequestError:
        """
        List genres
        """
        response = await self._request.make_request(
            "GET", url=f"{self._base_url}/api/genres", headers=self.headers
        )

        if not isinstance(response, RequestError):
            return [Genre.from_dict(s) for s in response]

        logger.debug(
            f"Bad Request(list): status - {response.status_code}: info - {str(response)}"
        )

        return response



__all__ = ["TooManyRequests", "RequestError"]


class TooManyRequests(Exception):
    """
    is a custom exception class used to represent the HTTP status code 429,
    which indicates that the client has sent too many requests in a given amount of time.
    """

    pass


class RequestError(Exception):
    """
    Custom exception class for handling errors that occur during HTTP requests.

    :param message: A string describing the error.
    :param status_code: (int, optional): The HTTP status code associated with the error. Defaults to None.
    """

    def __init__(self, message, status_code=None):
        """
        Initializes a new instance of the RequestError class.

        :param message: A string describing the error.
        :param status_code: (int, optional): The HTTP status code associated with the error. Defaults to None.
        """
        super().__init__(message)
        self.status_code = status_code



import logging
from typing import List

from shikimori.types.animes import (
    Relation,
)
from shikimori.types.externalLink import ExternalLink
from shikimori.types.franchise import Franchise
from shikimori.types.manga import Manga, MangaInfo
from shikimori.types.roles import Role
from shikimori.types.topics import Topic
from .base import BaseEndpoint
from ..exceptions import RequestError
from ..utils.filter import filter_none_parameters

logger = logging.getLogger(__name__)


class MangaEndpoint(BaseEndpoint):
    async def list(
        self,
        page: int = None,
        limit: int = None,
        order: str = None,
        kind: str = None,
        status: str = None,
        season: str = None,
        score: int = None,
        genre: str = None,
        publisher: str = None,
        franchise: str = None,
        censored: bool = None,
        mylist: str = None,
        ids: str = None,
        exclude_ids: str = None,
        search: str = None,
        genre_v2: str = None,
    ):
        """
        Most of parameters can be grouped in lists of values separated by comma:

        - season=2016,2015 – mangas with season 2016 year or with season 2015 year.

        - kind=manga,one_shot – mangas with kind Manga or with kind One Shot.

        Most of the parameters can be used in the subtraction mode:

        - season=!2016,!2015 – mangas without season 2016 year and without season 2015 year.

        - kind=!manga,!one_shot – mangas without kind Manga and without kind One Shot.

        Most of the parameters can be used in the combined mode:

        - season=2016,!summer_2016 – mangas with season 2016 year and without season summer_2016.

        :param page: Must be a number between 1 and 100000.
        :param limit: 50 maximum.
        :param order: Must be one of: id, id_desc, ranked, kind, popularity, name, aired_on, volumes, chapters, status, random, ranked_random, ranked_shiki, created_at, created_at_desc.
        :param kind: Must be one of: manga, manhwa, manhua, light_novel, novel, one_shot, doujin.
        :param status: Must be one of: anons, ongoing, released, paused, discontinued.
        :param season: summer_2017 spring_2016,fall_2016.
        :param score: must be a number.
        :param genre: List of genre ids separated by comma.
        :param publisher: List of publisher ids separated by comma.
        :param franchise: List of franchises separated by comma.
        :param censored: Set to false to allow hentai, yaoi and yuri.
        :param mylist: Must be one of: planned, watching, rewatching, completed, on_hold, dropped.
        :param ids: List of manga ids separated by comma
        :param exclude_ids: List of manga ids separated by comma
        :param search: Must be a String
        :param genre_v2: List of genre v2 ids separated by comma
        """

        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/mangas",
            params=filter_none_parameters(
                {
                    "page": page,
                    "limit": limit,
                    "order": order,
                    "kind": kind,
                    "status": status,
                    "season": season,
                    "score": score,
                    "genre": genre,
                    "publisher": publisher,
                    "franchise": franchise,
                    "censored": censored,
                    "mylist": mylist,
                    "ids": ids,
                    "exclude_ids": exclude_ids,
                    "search": search,
                    "genre_v2": genre_v2,
                }
            ),
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return [Manga.from_dict(manga) for manga in response]

        logger.debug(
            f"Bad Request(list): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def ById(self, id: int) -> MangaInfo | RequestError:
        """
        Show a manga.

        :param id: must be a number
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/mangas/{id}",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return MangaInfo.from_dict(response)

        logger.debug(
            f"Bad Request(ById): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def roles(self, id: int) -> List[Role] | RequestError:
        """
        list roles.

        :param id: must be a number
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/mangas/{id}/roles",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return [Role.from_dict(role) for role in response]

        logger.debug(
            f"Bad Request(roles): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def similar(self, id: int) -> List[Manga] | RequestError:
        """
        list similar.

        :param id: must be a number
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/mangas/{id}/similar",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return [Manga.from_dict(manga) for manga in response]

        logger.debug(
            f"Bad Request(similar): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def related(self, id: int) -> List[Relation] | RequestError:
        """
        list related.

        :param id: must be a number
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/mangas/{id}/related",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return [Relation.from_dict(relation) for relation in response]

        logger.debug(
            f"Bad Request(related): status - {response.status_code}: info - {str(response)}"
        )
        return response

    async def franchise(self, id: int) -> Franchise | RequestError:
        """
        list franchises.

        :param id: must be a number
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/mangas/{id}/franchise",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return Franchise.from_dict(response)

        logger.debug(
            f"Bad Request(franchise): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def ExternalLinks(self, id: int) -> List[ExternalLink] | RequestError:
        """
        list ExternalLinks.

        :param id: must be a number
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/mangas/{id}/external_links",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return [ExternalLink.from_dict(link) for link in response]

        logger.debug(
            f"Bad Request(externalLinks): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def topics(
        self,
        id: int,
        page: int = None,
        limit: int = None,
    ) -> List[Topic] | RequestError:
        """
        list topics.

        :param id: must be a number
        :param page: Must be a number between 1 and 100000.
        :param limit: 30 maximum
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/mangas/{id}/topics",
            headers=self.headers,
            params=filter_none_parameters({"page": page, "limit": limit}),
        )

        if not isinstance(response, RequestError):
            return [Topic.from_dict(topic) for topic in response]

        return response



import logging
from typing import List

from shikimori.types.videos import Video
from .base import BaseEndpoint
from ..exceptions import RequestError

logger = logging.getLogger(__name__)


class VideosEndpoint(BaseEndpoint):
    async def list(self, id: int) -> List[Video] | RequestError:
        """
        List videos.

        :param id: must be a number
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/animes/{id}/videos",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return [Video.from_dict(v) for v in response]

        logger.debug(
            f"Bad Request(list): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def create(
        self, id: int, kind: str, url: str, name: str
    ) -> Video | RequestError:
        """
        Create a video.

        Requires content oauth scope.

        :param id: must be a number
        :param kind: Must be one of: pv, character_trailer, cm, op, ed, op_ed_clip, clip, other, episode_preview.
        :param url: Supported hostings: YouTube,vk,ok,coub,rutube,vimeo,sibnet,yandex,streamable,smotret_anime,myvi,youmite,viuly,stormo,mediafile
        :param name: must be a string.
        """
        response = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/api/animes/{id}/videos",
            json={"video": {"url": url, "kind": kind, "name": name}},
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return Video.from_dict(response)

        logger.debug(
            f"Bad Request(create): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def delete(self, anime_id: int, id: int) -> None | RequestError:
        """
        Destroy a video.

        Requires content oauth scope.

        :param anime_id: must be a number
        :param id: must be a number
        """
        response = await self._request.make_request(
            "DELETE",
            url=f"{self._base_url}/api/animes/{anime_id}/videos/{id}",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return

        logger.debug(
            f"Bad Request(delete): status - {response.status_code}: info - {str(response)}"
        )

        return response



import logging

from .base import BaseEndpoint
from ..exceptions import RequestError
from ..utils.filter import filter_none_parameters

logger = logging.getLogger(__name__)


class AppearsEndpoint(BaseEndpoint):
    async def read(self, ids: str = None) -> None | RequestError:
        """
        Mark comments or topics as read.

        :param ids: str like 'comment-7,comment-8,topic-270101'.
        """

        response = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/api/appears",
            headers=self.headers,
            json=filter_none_parameters({"ids": ids}),
        )

        if not isinstance(response, RequestError):
            return

        logger.debug(
            f"Bad Request(mark): status - {response.status_code}: info - {str(response)}"
        )

        return response



from dataclasses import dataclass

from shikimori.utils.filter import handle_none_data
from shikimori.constants import SHIKIMORI_URL


@dataclass
class Photo:
    original: str
    preview: str
    x96: str
    x48: str

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            original=data.get("original"),
            preview=data.get("preview"),
            x96=data.get("x96"),
            x48=data.get("x48"),
        )

    @property
    def original_url(self) -> str:
        return f"{SHIKIMORI_URL}{self.original}"

    @property
    def preview_url(self) -> str:
        return f"{SHIKIMORI_URL}{self.preview}"

    @property
    def x96_url(self) -> str:
        return f"{SHIKIMORI_URL}{self.x96}"

    @property
    def x48_url(self) -> str:
        return f"{SHIKIMORI_URL}{self.x48}"


@dataclass
class PhotoExtended:
    x160: str
    x148: str
    x80: str
    x64: str
    x48: str
    x32: str
    x16: str

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            x160=data.get("x160"),
            x148=data.get("x148"),
            x80=data.get("x80"),
            x64=data.get("x64"),
            x48=data.get("x48"),
            x32=data.get("x32"),
            x16=data.get("x16"),
        )

    @property
    def x160_url(self) -> str:
        return f"{SHIKIMORI_URL}{self.x160}"

    @property
    def x148_url(self) -> str:
        return f"{SHIKIMORI_URL}{self.x148}"

    @property
    def x80_url(self) -> str:
        return f"{SHIKIMORI_URL}{self.x80}"

    @property
    def x64_url(self) -> str:
        return f"{SHIKIMORI_URL}{self.x64}"

    @property
    def x48_url(self) -> str:
        return f"{SHIKIMORI_URL}{self.x48}"

    @property
    def x32_url(self) -> str:
        return f"{SHIKIMORI_URL}{self.x32}"

    @property
    def x16_url(self) -> str:
        return f"{SHIKIMORI_URL}{self.x16}"


@dataclass
class ClubImage:
    id: int
    original_url: str
    main_url: str
    preview_url: str
    can_destroy: bool
    user_id: int

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            id=data.get("id"),
            original_url=data.get("original_url"),
            main_url=data.get("main_url"),
            preview_url=data.get("preview_url"),
            can_destroy=data.get("can_destroy"),
            user_id=data.get("user_id"),
        )

    @property
    def original_full_url(self) -> str:
        return f"{SHIKIMORI_URL}{self.original_url}"

    @property
    def main_full_url(self) -> str:
        return f"{SHIKIMORI_URL}{self.main_url}"

    @property
    def preview_full_url(self) -> str:
        return f"{SHIKIMORI_URL}{self.preview_url}"


@dataclass
class UserImage:
    id: int
    preview: str
    url: str
    bbcode: str

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            id=data.get("id"),
            preview=data.get("preview"),
            url=data.get("url"),
            bbcode=data.get("bbcode"),
        )

    @property
    def preview_url(self) -> str:
        return f"{SHIKIMORI_URL}{self.preview}"

    @property
    def full_url(self) -> str:
        return f"{SHIKIMORI_URL}{self.url}"


@dataclass
class Logo:
    original: str
    main: str
    x96: str
    x73: str
    x48: str

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            original=data.get("original"),
            main=data.get("main"),
            x96=data.get("x96"),
            x73=data.get("x73"),
            x48=data.get("x48"),
        )

    @property
    def original_url(self) -> str:
        return f"{SHIKIMORI_URL}{self.original}"

    @property
    def main_url(self) -> str:
        return f"{SHIKIMORI_URL}{self.main}"

    @property
    def x96_url(self) -> str:
        return f"{SHIKIMORI_URL}{self.x96}"

    @property
    def x73_url(self) -> str:
        return f"{SHIKIMORI_URL}{self.x73}"

    @property
    def x48_url(self) -> str:
        return f"{SHIKIMORI_URL}{self.x48}"



from .abuse_requests import AbuseRequestEndpoint
from .achievements import AchievementsEndpoint
from .animes import AnimeEndpoint
from .appears import AppearsEndpoint
from .bans import BanEndpoint
from .base import BaseEndpoint
from .calendar import CalendarEndpoint
from .characters import CharacterEndpoint
from .club import ClubEndpoint
from .comments import CommentEndpoint
from .constants import ConstantsEndpoint
from .dialogs import DialogsEndpoint
from .episode_notifications import EpisodeNotificationEndpoint
from .favorites import FavoritesEndpoint
from .forums import ForumEndpoint
from .friends import FriendEndpoint
from .genres import GenreEndpoint
from .manga import MangaEndpoint
from .message import MessageEndpoint
from .people import PeopleEndpoint
from .publisher import PublisherEndpoint
from .ranobe import RanobeEndpoint
from .review import ReviewEndpoint
from .stats import StatsEndpoint
from .studios import StudiosEndpoint
from .styles import StylesEndpoint
from .topic_ignore import TopicIgnoreEndpoint
from .topics import TopicsEndpoint
from .user_ignore import UserIgnoreEndpoint
from .user_images import UserImageEndpoint
from .user_rates import UserRatesEndpoint
from .users import UserEndpoint
from .videos import VideosEndpoint
from .grapql import GraphQlEndpoint

__all__ = [
    "AbuseRequestEndpoint",
    "GraphQlEndpoint",
    "AchievementsEndpoint",
    "AnimeEndpoint",
    "AppearsEndpoint",
    "BanEndpoint",
    "BaseEndpoint",
    "CalendarEndpoint",
    "CharacterEndpoint",
    "ClubEndpoint",
    "CommentEndpoint",
    "ConstantsEndpoint",
    "DialogsEndpoint",
    "EpisodeNotificationEndpoint",
    "FavoritesEndpoint",
    "ForumEndpoint",
    "FriendEndpoint",
    "GenreEndpoint",
    "MangaEndpoint",
    "MessageEndpoint",
    "PeopleEndpoint",
    "PublisherEndpoint",
    "RanobeEndpoint",
    "ReviewEndpoint",
    "StatsEndpoint",
    "StudiosEndpoint",
    "StylesEndpoint",
    "TopicIgnoreEndpoint",
    "TopicsEndpoint",
    "UserIgnoreEndpoint",
    "UserImageEndpoint",
    "UserRatesEndpoint",
    "UserEndpoint",
    "VideosEndpoint",
]



from dataclasses import dataclass

from shikimori.utils.filter import handle_none_data


@dataclass
class Style:
    id: int
    owner_id: int
    owner_type: str
    name: str
    css: str
    compiled_css: None
    created_at: str
    updated_at: str

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            id=data.get("id"),
            owner_id=data.get("owner_id"),
            owner_type=data.get("owner_type"),
            name=data.get("name"),
            css=data.get("css"),
            compiled_css=None,
            created_at=data.get("created_at"),
            updated_at=data.get("updated_at"),
        )



import logging

from shikimori.types.user_ignore import UserIgnore
from .base import BaseEndpoint
from ..exceptions import RequestError

logger = logging.getLogger(__name__)


class UserIgnoreEndpoint(BaseEndpoint):
    async def ignore(self, id: str | int) -> UserIgnore | RequestError:
        """
        ignore user.

        Requires ignores oauth scope.

        :param id: must be a number
        """

        response = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/api/v2/users/{id}/ignore",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return UserIgnore.from_dict(response)

        logger.debug(
            f"Bad Request(ignore): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def unignore(self, id: str | int) -> UserIgnore | RequestError:
        """
        unignore user.

        Requires ignores oauth scope.

        :param id: must be a number
        """

        response = await self._request.make_request(
            "DELETE",
            url=f"{self._base_url}/api/v2/users/{id}/ignore",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return UserIgnore.from_dict(response)

        logger.debug(
            f"Bad Request(unignore): status - {response.status_code}: info - {str(response)}"
        )

        return response



import logging

from shikimori.types.user_rates import UserRate
from .base import BaseEndpoint
from ..exceptions import RequestError
from ..utils.filter import filter_none_parameters

logger = logging.getLogger(__name__)


class UserRatesEndpoint(BaseEndpoint):
    async def ById(self, id: int) -> UserRate | RequestError:
        """
        Show a user rate.

        :param id: must be a number
        """

        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/v2/user_rates/{id}",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return UserRate.from_dict(response)

        logger.debug(
            f"Bad Request(get_user_rate): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def list(
        self,
        user_id: int = None,
        target_id: int = None,
        target_type: str = None,
        status: str = None,
        page: int = None,
        limit: int = None,
    ) -> list[UserRate] | RequestError:
        """
        List user rates.

        :param user_id: must be a number
        :param target_id: must be a number
        :param target_type: Must be one of: Anime, Manga
        :param status: must be one of: planned, watching, rewatching, completed, on_hold, dropped
        :param page: must be a number
        :param limit: must be a number (Maximum - 1000)
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/v2/user_rates",
            params=filter_none_parameters(
                {
                    "user_id": user_id,
                    "target_id": target_id,
                    "target_type": target_type,
                    "status": status,
                    "page": page,
                    "limit": limit,
                }
            ),
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return [UserRate.from_dict(user_rate) for user_rate in response]

        logger.debug(
            f"Bad Request(get_user_rate): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def create(
        self,
        user_id: int,
        target_id: int,
        target_type: str,
        status: str | None = None,
        score: int | None = None,
        chapters: int | None = None,
        episodes: int | None = None,
        volumes: int | None = None,
        rewatches: int | None = None,
        text: str | None = None,
    ) -> UserRate | RequestError:
        """
        Create a user rate.

        Requires user_rates oauth scope.

        :param text: Must be a String
        :param rewatches: Must be a number.
        :param volumes: Must be a number.
        :param episodes: Must be a number.
        :param chapters: Must be a number.
        :param score: Must be a number.
        :param status: Must be one of: planned, watching, rewatching, completed, on_hold, dropped
        :param target_type:  Must be one of: Anime, Manga.
        :param target_id: Must be a number.
        :param user_id: Must be a number.
        """
        response = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/api/v2/user_rates",
            json={
                "user_rate": filter_none_parameters(
                    {
                        "user_id": user_id,
                        "target_id": target_id,
                        "target_type": target_type,
                        "status": status,
                        "chapters": chapters,
                        "volumes": volumes,
                        "rewatches": rewatches,
                        "text": text,
                        "score": score,
                        "episodes": episodes,
                    }
                )
            },
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return UserRate.from_dict(response)

        logger.debug(
            f"Bad Request(create_user_rate): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def update(
        self,
        user_rate_id: int,
        status: str | None = None,
        score: int | None = None,
        chapters: int | None = None,
        episodes: int | None = None,
        volumes: int | None = None,
        rewatches: int | None = None,
        text: str | None = None,
    ) -> UserRate | RequestError:
        """
        Update a user rate.

        Requires user_rates oauth scope.

        :param user_rate_id: Must be a number.
        :param status: Must be a number.
        :param score: Must be a number.
        :param chapters: Must be a number.
        :param episodes: Must be a number.
        :param volumes: Must be a number.
        :param rewatches: Must be a number.
        :param text: Must be a String
        """
        response = await self._request.make_request(
            "PATCH",
            url=f"{self._base_url}/api/v2/user_rates/{user_rate_id}",
            body={
                "user_rate": filter_none_parameters(
                    {
                        "status": status,
                        "score": score,
                        "chapters": chapters,
                        "episodes": episodes,
                        "volumes": volumes,
                        "rewatches": rewatches,
                        "text": text,
                    }
                )
            },
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return UserRate.from_dict(response)

        logger.debug(
            f"Bad Request(update_user_rate): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def increment(self, id: int) -> UserRate | RequestError:
        """
        requires user_rates oauth scope.

        Increment episodes/chapters by 1.

        :param id:  Must be a number.
        """
        response = await self._request.make_request(
            "POST",
            url=f"{self._base_url}/api/v2/user_rates/{id}/increment",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return UserRate.from_dict(response)

        logger.debug(
            f"Bad Request(increment): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def delete(self, id: int) -> None | RequestError:
        """
        Destroy a user rate.

        requires user_rates oauth scope.

        :param id: must be a number.
        """

        response = await self._request.make_request(
            "DELETE",
            url=f"{self._base_url}/api/v2/user_rates/{id}",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return

        logger.debug(
            f"Bad Request(delete): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def cleanup(self, type: str) -> dict | RequestError:
        """
        Be careful to use.

        :param type: Must be one of: anime, manga.
        """
        response = await self._request.make_request(
            "DELETE",
            url=f"{self._base_url}/api/user_rates/{type}/cleanup",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return response

        logger.debug(
            f"Bad Request(cleanup): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def reset(self, type: str) -> dict | RequestError:
        """
        Be careful to use.

        Reset all user scores to 0.

        :param type: Must be one of: anime, manga.
        """
        response = await self._request.make_request(
            "DELETE",
            url=f"{self._base_url}/api/user_rates/{type}/reset",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return response

        logger.debug(
            f"Bad Request(reset): status - {response.status_code}: info - {str(response)}"
        )

        return response



import logging
from typing import List

from shikimori.types.animes import (
    Relation,
)
from shikimori.types.externalLink import ExternalLink
from shikimori.types.franchise import Franchise
from shikimori.types.ranobe import Ranobe, RanobeInfo
from shikimori.types.roles import Role
from shikimori.types.topics import Topic
from .base import BaseEndpoint
from ..exceptions import RequestError
from ..utils.filter import filter_none_parameters

logger = logging.getLogger(__name__)


class RanobeEndpoint(BaseEndpoint):
    async def list(
        self,
        page: int = None,
        limit: int = None,
        order: str = None,
        status: str = None,
        season: str = None,
        score: int = None,
        genre: str = None,
        publisher: str = None,
        franchise: str = None,
        censored: bool = None,
        mylist: str = None,
        ids: str = None,
        exclude_ids: str = None,
        search: str = None,
    ) -> List[Ranobe] | RequestError:
        """
        Most of parameters can be grouped in lists of values separated by comma:

        - season=2016,2015 – ranobe with season 2016 year or with season 2015 year

        Most of the parameters can be used in the subtraction mode:

        - season=!2016,!2015 – ranobe without season 2016 year and without season 2015 year

        Most of the parameters can be used in the combined mode:

        - season=2016,!summer_2016 – ranobe with season 2016 year and without season summer_2016.

        :param page: Must be a number between 1 and 100000.
        :param limit: 50 maximum.
        :param order: Must be one of: id, id_desc, ranked, kind, popularity, name, aired_on, volumes, chapters, status, random, ranked_random, ranked_shiki, created_at, created_at_desc.
        :param status: Must be one of: anons, ongoing, released, paused, discontinued.
        :param season: summer_2017 spring_2016,fall_2016.
        :param score: must be a number.
        :param genre: List of genre ids separated by comma.
        :param publisher: List of publisher ids separated by comma.
        :param franchise: List of franchises separated by comma.
        :param censored: Set to false to allow hentai, yaoi and yuri.
        :param mylist: Must be one of: planned, watching, rewatching, completed, on_hold, dropped.
        :param ids: List of ranobe ids separated by comma.
        :param exclude_ids: List of ranobe ids separated by comma.
        :param search: Must be a String.
        """

        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/ranobe",
            params=filter_none_parameters(
                {
                    "page": page,
                    "limit": limit,
                    "order": order,
                    "status": status,
                    "season": season,
                    "score": score,
                    "genre": genre,
                    "publisher": publisher,
                    "franchise": franchise,
                    "censored": censored,
                    "mylist": mylist,
                    "ids": ids,
                    "exclude_ids": exclude_ids,
                    "search": search,
                }
            ),
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return [Ranobe.from_dict(ranobe) for ranobe in response]

        logger.debug(
            f"Bad Request(list): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def ById(self, id: int) -> RanobeInfo | RequestError:
        """
        Show a ranobe.

        :param id: must be a number
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/ranobe/{id}",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return RanobeInfo.from_dict(response)

        logger.debug(
            f"Bad Request(ById): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def roles(self, id: int) -> List[Role] | RequestError:
        """
        list roles.

        :param id: must be a number
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/ranobe/{id}/roles",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return [Role.from_dict(role) for role in response]

        logger.debug(
            f"Bad Request(roles): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def similar(self, id: int) -> List[Ranobe] | RequestError:
        """
        list similar.

        :param id: must be a number
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/ranobe/{id}/similar",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return [Ranobe.from_dict(ranobe) for ranobe in response]

        logger.debug(
            f"Bad Request(similar): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def related(self, id: int) -> List[Relation] | RequestError:
        """
        list related.

        :param id: must be a number
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/ranobe/{id}/related",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return [Relation.from_dict(relation) for relation in response]

        logger.debug(
            f"Bad Request(related): status - {response.status_code}: info - {str(response)}"
        )
        return response

    async def franchise(self, id: int) -> Franchise | RequestError:
        """
        list franchise.

        :param id: must be a number
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/ranobe/{id}/franchise",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return Franchise.from_dict(response)

        logger.debug(
            f"Bad Request(franchise): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def ExternalLinks(self, id: int) -> List[ExternalLink] | RequestError:
        """
        list ExternalLinks.

        :param id: must be a number
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/ranobe/{id}/external_links",
            headers=self.headers,
        )

        if not isinstance(response, RequestError):
            return [ExternalLink.from_dict(link) for link in response]

        logger.debug(
            f"Bad Request(externalLinks): status - {response.status_code}: info - {str(response)}"
        )

        return response

    async def topics(
        self,
        id: int,
        page: int = None,
        limit: int = None,
    ) -> List[Topic] | RequestError:
        """
        list ExternalLinks.

        :param limit: 30 maximum.
        :param page: Must be a number between 1 and 100000.
        :param id: must be a number
        """
        response = await self._request.make_request(
            "GET",
            url=f"{self._base_url}/api/ranobe/{id}/topics",
            headers=self.headers,
            params=filter_none_parameters({"page": page, "limit": limit}),
        )

        if not isinstance(response, RequestError):
            return [Topic.from_dict(topic) for topic in response]

        logger.debug(
            f"Bad Request(topics): status - {response.status_code}: info - {str(response)}"
        )

        return response



from dataclasses import dataclass
from shikimori.types.user import User
from .message import Message
from ..utils.filter import handle_none_data


@dataclass
class Dialog:
    target_user: User
    message: Message

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            target_user=User.from_dict(data.get("target_user")),
            message=Message.from_dict(data.get("message")),
        )



from shikimori.types.user import User
from shikimori.types.photo import Photo
from dataclasses import dataclass
from shikimori.utils.filter import handle_none_data


@dataclass
class Title:
    id: int
    name: str
    russian: str
    image: Photo
    url: str
    kind: str
    score: str
    status: str
    aired_on: str
    released_on: str
    episodes: int = 0
    episodes_aired: int = 0
    volumes: int = 0
    chapters: int = 0

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            episodes=data.get("episodes", 0),
            episodes_aired=data.get("episodes_aired", 0),
            volumes=data.get("volumes", 0),
            chapters=data.get("chapters", 0),
            id=data["id"],
            name=data["name"],
            russian=data["russian"],
            image=Photo.from_dict(data.get("image")),
            url=data["url"],
            kind=data["kind"],
            score=data["score"],
            status=data["status"],
            aired_on=data["aired_on"],
            released_on=data["released_on"],
        )


@dataclass
class Forum:
    id: int
    position: int
    name: str
    permalink: str
    url: str

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            id=data.get("id"),
            position=data.get("position"),
            name=data.get("name"),
            permalink=data.get("permalink"),
            url=data.get("url"),
        )


@dataclass
class Linked:
    id: int
    name: str
    russian: str
    image: Photo
    url: str
    kind: str
    score: str
    status: str
    aired_on: str | None
    released_on: str | None
    episodes: int = None
    episodes_aired: int = None
    volumes: int = None
    chapters: int = None

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            id=data.get("id"),
            name=data.get("name"),
            russian=data.get("russian"),
            image=Photo.from_dict(data.get("image")),
            url=data.get("url"),
            kind=data.get("kind"),
            score=data.get("score"),
            status=data.get("status"),
            aired_on=data.get("aired_on"),
            released_on=data.get("released_on"),
            episodes=data.get("episodes"),
            episodes_aired=data.get("episodes_aired"),
            volumes=data.get("volumes"),
            chapters=data.get("chapters"),
        )


@dataclass
class Topic:
    id: int
    topic_title: str
    body: str
    html_body: str
    html_footer: str
    created_at: str
    comments_count: int
    forum: Forum
    user: User
    type: str
    linked_id: int
    linked_type: str
    linked: Linked | None
    viewed: bool
    last_comment_viewed: bool
    event: str
    episode: int

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            id=data.get("id"),
            topic_title=data.get("topic_title"),
            body=data.get("body"),
            html_body=data.get("html_body"),
            html_footer=data.get("html_footer"),
            created_at=data.get("created_at"),
            comments_count=data.get("comments_count"),
            forum=Forum.from_dict(data.get("forum")),
            user=User.from_dict(data.get("user")),
            type=data.get("type"),
            linked_id=data.get("linked_id"),
            linked_type=data.get("linked_type"),
            linked=Linked.from_dict(data.get("linked")) if data.get("linked") else None,
            viewed=data.get("viewed"),
            last_comment_viewed=data.get("last_comment_viewed"),
            event=data.get("event"),
            episode=data.get("episode"),
        )


@dataclass
class Status:
    id: int
    linked: Linked
    event: str
    episode: int
    created_at: str
    url: str

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            id=data.get("id"),
            linked=Linked.from_dict(data.get("linked")),
            event=data.get("event"),
            episode=data.get("episode"),
            created_at=data.get("created_at"),
            url=data.get("url"),
        )


@dataclass
class ReviewLinked:
    id: int
    user: User
    target: Title
    votes_count: int
    votes_for: int
    body: str
    html_body: str
    overall: int
    storyline: int
    music: int
    characters: int
    animation: int
    created_at: str

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            id=data.get("id"),
            user=User.from_dict(data.get("user")),
            target=Title.from_dict(data.get("target")),
            votes_count=data.get("votes_count"),
            votes_for=data.get("votes_for"),
            body=data.get("body"),
            html_body=data.get("html_body"),
            overall=data.get("overall"),
            storyline=data.get("storyline"),
            music=data.get("music"),
            characters=data.get("characters"),
            animation=data.get("animation"),
            created_at=data.get("created_at"),
        )


@dataclass
class TopicReview:
    id: int
    topic_title: str
    body: str
    html_body: str
    html_footer: str
    created_at: str
    comments_count: int
    forum: Forum
    user: User
    type: str
    linked_id: int
    linked_type: str
    linked: Linked | None
    viewed: bool
    last_comment_viewed: bool
    event: str
    episode: int
    linked: ReviewLinked | None

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            id=data.get("id"),
            topic_title=data.get("topic_title"),
            body=data.get("body"),
            html_body=data.get("html_body"),
            html_footer=data.get("html_footer"),
            created_at=data.get("created_at"),
            comments_count=data.get("comments_count"),
            forum=Forum.from_dict(data.get("forum")),
            user=User.from_dict(data.get("user")),
            type=data.get("type"),
            linked_id=data.get("linked_id"),
            linked_type=data.get("linked_type"),
            linked=ReviewLinked.from_dict(data.get("linked")),
            viewed=data.get("viewed"),
            last_comment_viewed=data.get("last_comment_viewed"),
            event=data.get("event"),
            episode=data.get("episode"),
        )



from shikimori.types.manga import Manga
from shikimori.types.animes import Anime
from shikimori.types.character import CharacterBrief
from shikimori.types.user import User
from .photo import ClubImage, Logo
from .topics import Topic
from dataclasses import dataclass

from ..utils.filter import handle_none_data


@dataclass
class Club:
    id: int
    name: str
    logo: Logo
    is_censored: bool
    join_policy: str
    comment_policy: str

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            id=data.get("id"),
            name=data.get("name"),
            logo=Logo.from_dict(data.get("logo")),
            is_censored=data.get("is_censored"),
            join_policy=data.get("join_policy"),
            comment_policy=data.get("comment_policy"),
        )


@dataclass
class ClubInfo(Club):
    description: str
    description_html: str
    mangas: list[Manga]
    characters: list[CharacterBrief]
    thread_id: int
    topic_id: int
    user_role: str
    style_id: int
    members: list[User]
    animes: list[Anime]
    images: list[ClubImage]

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            description=data.get("description"),
            description_html=data.get("description_html"),
            mangas=[
                Manga.from_dict(manga_data) for manga_data in data.get("mangas", [])
            ],
            characters=[
                CharacterBrief.from_dict(character_data)
                for character_data in data.get("characters", [])
            ],
            thread_id=data.get("thread_id"),
            topic_id=data.get("topic_id"),
            user_role=data.get("user_role"),
            style_id=data.get("style_id"),
            members=[
                User.from_dict(member_data) for member_data in data.get("members", [])
            ],
            animes=[
                Anime.from_dict(anime_data) for anime_data in data.get("animes", [])
            ],
            images=[
                ClubImage.from_dict(image_data) for image_data in data.get("images", [])
            ],
            id=data.get("id"),
            name=data.get("name"),
            logo=Logo.from_dict(data.get("logo")),
            is_censored=data.get("is_censored"),
            join_policy=data.get("join_policy"),
            comment_policy=data.get("comment_policy"),
        )


@dataclass
class Collection(Topic):
    pass



from dataclasses import dataclass

from shikimori.utils.filter import handle_none_data


@dataclass
class Studio:
    id: int
    name: str
    filtered_name: str
    real: bool
    image: str

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            id=data.get("id"),
            name=data.get("name"),
            filtered_name=data.get("filtered_name"),
            real=data.get("real"),
            image=data.get("image"),
        )



from dataclasses import dataclass

from shikimori.utils.filter import handle_none_data


@dataclass
class Node:
    id: int
    date: int
    name: str
    image_url: str
    url: str
    year: int | None
    kind: str
    weight: int

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            id=data.get("id"),
            date=data.get("date"),
            name=data.get("name"),
            image_url=data.get("image_url"),
            url=data.get("url"),
            year=data.get("year") if data.get("year") is not None else None,
            kind=data.get("kind"),
            weight=data.get("weight"),
        )


@dataclass
class Link:
    id: int
    source_id: int
    source: int
    target_id: int
    target: int
    weight: int
    relation: str

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            id=data.get("id"),
            source_id=data.get("source_id"),
            source=data.get("source"),
            target_id=data.get("target_id"),
            target=data.get("target"),
            weight=data.get("weight"),
            relation=data.get("relation"),
        )


@dataclass
class Franchise:
    nodes: list[Node]
    links: list[Link]
    current_id: int

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            nodes=[Node.from_dict(node_data) for node_data in data.get("nodes", [])],
            links=[Link.from_dict(link_data) for link_data in data.get("links", [])],
            current_id=data.get("current_id"),
        )



from dataclasses import dataclass

from shikimori.utils.filter import handle_none_data


@dataclass
class Genre:
    id: int
    name: str
    russian: str
    kind: str

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            id=data.get("id"),
            name=data.get("name"),
            russian=data.get("russian"),
            kind=data.get("kind"),
        )


@dataclass
class GenreExtended:
    id: int
    name: str
    russian: str
    kind: str
    entry_type: str

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            id=data.get("id"),
            name=data.get("name"),
            russian=data.get("russian"),
            kind=data.get("kind"),
            entry_type=data.get("entry_type"),
        )



from .types import *
from .client import *
from .constants import *
from .exceptions import *
from .requestLimiter import *
from .endpoints import *
from .auth import *


__all__ = ["Shikimori", "types"]



from dataclasses import dataclass

from shikimori.utils.filter import handle_none_data


@dataclass
class UserRateBrief:
    id: int
    score: int
    status: str
    rewatches: int
    episodes: int
    volumes: int
    chapters: int
    text: str
    text_html: str
    created_at: str
    updated_at: str

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            id=data.get("id"),
            score=data.get("score"),
            status=data.get("status"),
            rewatches=data.get("rewatches"),
            episodes=data.get("episodes"),
            volumes=data.get("volumes"),
            chapters=data.get("chapters"),
            text=data.get("text"),
            text_html=data.get("text_html"),
            created_at=data.get("created_at"),
            updated_at=data.get("updated_at"),
        )


@dataclass
class UserRate:
    id: int
    user_id: int
    target_id: int
    target_type: str
    score: int
    status: str
    rewatches: int
    episodes: int
    volumes: int
    chapters: int
    text: str
    text_html: str
    created_at: str
    updated_at: str

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            user_id=data.get("user_id"),
            id=data.get("id"),
            target_id=data.get("target_id"),
            target_type=data.get("target_type"),
            score=data.get("score"),
            status=data.get("status"),
            rewatches=data.get("rewatches"),
            episodes=data.get("episodes"),
            volumes=data.get("volumes"),
            chapters=data.get("chapters"),
            text=data.get("text"),
            text_html=data.get("text_html"),
            created_at=data.get("created_at"),
            updated_at=data.get("updated_at"),
        )



from .requestLimiter import RequestLimiter
from .constants import MAX_REQUESTS_PER_SECOND, SHIKIMORI_URL, MAX_REQUESTS_PER_MINUTE
from .request import Request
from shikimori.auth import Auth
from shikimori.types.auth import AuthOptions
from shikimori import endpoints
from logging import basicConfig, DEBUG

__all__ = ["Shikimori"]


class Shikimori:
    """
    Main class for interacting with the Shikimori API.

    This class provides methods and endpoints for working with various features of the Shikimori API,
    such as anime, manga, users, forums, etc.
    """

    def __init__(
        self,
        *,
        user_agent: str = None,
        client_id: str = None,
        client_secret: str = None,
        redirect_uri: str = "urn:ietf:wg:oauth:2.0:oob",
        scopes: list[str] = None,
        base_url: str = None,
        logging: int | bool = None,
    ):
        """
        Initialize client for working with the Shikimori API.

        :param user_agent: User-agent string to be used in API requests.
        :param client_id: Client ID for OAuth authentication.
        :param client_secret: Client secret for OAuth authentication.
        :param scopes: Scopes for OAuth authentication.
        :param redirect_uri: Redirect URI for OAuth authentication. Defaults to "urn:ietf:wg:oauth:2.0:oob".
        :param base_url: Base URL for the Shikimori API. Defaults to None.
        :param logging: Logging level for debug information. If True, debug logging is enabled. If False, logging is disabled. Defaults to None.

        Raises:
            ValueError: If user_agent is not specified.
        """

        if not user_agent:
            raise ValueError("You need to specify user-agent")

        if logging is not None:
            if logging is True:
                basicConfig(level=DEBUG)
            else:
                basicConfig(level=logging)

        # dependencies
        self._base_url = base_url or SHIKIMORI_URL
        self._request = Request()
        self._limiter = RequestLimiter(
            MAX_REQUESTS_PER_SECOND,
            MAX_REQUESTS_PER_MINUTE,
            self._request,
        )

        # Auth dependencies
        self._options = AuthOptions(
            client_id=client_id,
            redirect_uri=redirect_uri,
            client_secret=client_secret,
            scopes=scopes,
        )
        self._user_agent = user_agent

        # dependencies
        self._deps = {
            "base_url": self._base_url,
            "request": self._limiter,
            "user_agent": self._user_agent,
        }

        self.abuseRequest = endpoints.AbuseRequestEndpoint(**self._deps)
        """Endpoint for making requests to the URL 'https://shikimori.one/api/v2/abuse_requests/'"""
        self.achievement = endpoints.AchievementsEndpoint(**self._deps)
        """Endpoint for making requests to the URL 'https://shikimori.one/api/achievements'"""
        self.anime = endpoints.AnimeEndpoint(**self._deps)
        """Endpoint for making requests to the URL 'https://shikimori.one/api/animes'"""
        self.appear = endpoints.AppearsEndpoint(**self._deps)
        """Endpoint for making requests to the URL 'https://shikimori.one/api/appears'"""
        self.ban = endpoints.BanEndpoint(**self._deps)
        """Endpoint for making requests to the URL 'https://shikimori.one/api/bans'"""
        self.calendar = endpoints.CalendarEndpoint(**self._deps)
        """Endpoint for making requests to the URL 'https://shikimori.one/api/calendar'"""
        self.character = endpoints.CharacterEndpoint(**self._deps)
        """Endpoint for making requests to the URL 'https://shikimori.one/api/characters/'"""
        self.club = endpoints.ClubEndpoint(**self._deps)
        """Endpoint for making requests to the URL 'https://shikimori.one/api/clubs'"""
        self.comment = endpoints.CommentEndpoint(**self._deps)
        """Endpoint for making requests to the URL 'https://shikimori.one/api/comments'"""
        self.constant = endpoints.ConstantsEndpoint(**self._deps)
        """Endpoint for making requests to the URL 'https://shikimori.one/api/constants/anime'"""
        self.dialog = endpoints.DialogsEndpoint(**self._deps)
        """Endpoint for making requests to the URL 'https://shikimori.one/api/dialogs'"""
        self.episodeNotification = endpoints.EpisodeNotificationEndpoint(**self._deps)
        """Endpoint for making requests to the URL 'https://shikimori.one/api/v2/episode_notifications'"""
        self.favorite = endpoints.FavoritesEndpoint(**self._deps)
        """Endpoint for making requests to the URL 'https://shikimori.one/api/favorites/'"""
        self.forum = endpoints.ForumEndpoint(**self._deps)
        """Endpoint for making requests to the URL 'https://shikimori.one/api/forums'"""
        self.friend = endpoints.FriendEndpoint(**self._deps)
        """Endpoint for making requests to the URL 'https://shikimori.one/api/friends/:id'"""
        self.genre = endpoints.GenreEndpoint(**self._deps)
        """Endpoint for making requests to the URL 'https://shikimori.one/api/genres'"""
        self.manga = endpoints.MangaEndpoint(**self._deps)
        """Endpoint for making requests to the URL 'https://shikimori.one/api/mangas'"""
        self.message = endpoints.MessageEndpoint(**self._deps)
        """Endpoint for making requests to the URL 'https://shikimori.one/api/messages/'"""
        self.people = endpoints.PeopleEndpoint(**self._deps)
        """Endpoint for making requests to the URL 'https://shikimori.one/api/people/:id'"""
        self.publisher = endpoints.PublisherEndpoint(**self._deps)
        """Endpoint for making requests to the URL 'https://shikimori.one/api/publishers'"""
        self.ranobe = endpoints.RanobeEndpoint(**self._deps)
        """Endpoint for making requests to the URL 'https://shikimori.one/api/ranobe'"""
        self.review = endpoints.ReviewEndpoint(**self._deps)
        """Endpoint for making requests to the URL 'https://shikimori.one/api/reviews'"""
        self.stats = endpoints.StatsEndpoint(**self._deps)
        """Endpoint for making requests to the URL 'https://shikimori.one/api/stats/active_users'"""
        self.studio = endpoints.StudiosEndpoint(**self._deps)
        """Endpoint for making requests to the URL 'https://shikimori.one/api/studios'"""
        self.style = endpoints.StylesEndpoint(**self._deps)
        """Endpoint for making requests to the URL 'https://shikimori.one/api/styles/'"""
        self.topicIgnore = endpoints.TopicIgnoreEndpoint(**self._deps)
        """Endpoint for making requests to the URL 'https://shikimori.one/api/v2/topics/'"""
        self.topic = endpoints.TopicsEndpoint(**self._deps)
        """Endpoint for making requests to the URL 'https://shikimori.one/api/topics'"""
        self.userIgnore = endpoints.UserIgnoreEndpoint(**self._deps)
        """Endpoint for making requests to the URL 'https://shikimori.one/api/v2/users/:user_id/ignore'"""
        self.userImage = endpoints.UserImageEndpoint(**self._deps)
        """Endpoint for making requests to the URL 'https://shikimori.one/api/user_images'"""
        self.userRate = endpoints.UserRatesEndpoint(**self._deps)
        """Endpoint for making requests to the URL 'https://shikimori.one/api/v2/user_rates/'"""
        self.user = endpoints.UserEndpoint(**self._deps)
        """Endpoint for making requests to the URL 'https://shikimori.one/api/users'"""
        self.video = endpoints.VideosEndpoint(**self._deps)
        """Endpoint for making requests to the URL 'https://shikimori.one/api/animes/:anime_id/videos'"""
        self.auth = Auth(self._limiter, self._user_agent, self._options, self._base_url)
        """Endpoint for OAUTH"""
        self.graphql = endpoints.GraphQlEndpoint(**self._deps)
        """Endpoint for making requests to the URL 'https://shikimori.one/api/grapql with your fields'"""

    def set_token(self, token: str) -> None:
        """
        Set OAuth token for authentication.

        :param token: OAuth token to be set.
        """
        self._request.set_token(token)



from dataclasses import dataclass
from shikimori.types.user import User
from shikimori.types.topics import Linked
from shikimori.utils.filter import handle_none_data


@dataclass
class Message:
    id: int
    kind: str
    read: bool
    body: str
    html_body: str
    created_at: str
    linked_id: int | None
    linked_type: str | None
    linked: Linked | None

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            id=data.get("id"),
            kind=data.get("kind"),
            read=data.get("read"),
            body=data.get("body"),
            html_body=data.get("html_body"),
            created_at=data.get("created_at"),
            linked_id=data.get("linked_id"),
            linked_type=data.get("linked_type"),
            linked=Linked.from_dict(data.get("linked")) if data.get("linked") else None,
        )


@dataclass
class MessageInfo:
    id: int
    kind: str
    read: bool
    body: str
    html_body: str
    created_at: str
    linked_id: int | None
    linked_type: str | None
    linked: Linked | None
    sender: User
    to: User

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            id=data.get("id"),
            kind=data.get("kind"),
            read=data.get("read"),
            body=data.get("body"),
            html_body=data.get("html_body"),
            created_at=data.get("created_at"),
            linked_id=data.get("linked_id"),
            linked_type=data.get("linked_type"),
            linked=Linked.from_dict(data.get("linked")) if data.get("linked") else None,
            sender=User.from_dict(data.get("from")),
            to=User.from_dict(data.get("to")),
        )



from dataclasses import dataclass

from shikimori.utils.filter import handle_none_data


@dataclass
class Publisher:
    id: int
    name: str

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(id=data.get("id"), name=data.get("name"))



from dataclasses import dataclass

from .character import CharacterBrief
from ..utils.filter import handle_none_data


@dataclass
class Role:
    roles: list[str]
    roles_russian: list[str]
    character: CharacterBrief
    person: CharacterBrief

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            roles=data.get("roles", []),
            roles_russian=data.get("roles_russian", []),
            character=(
                CharacterBrief.from_dict(character)
                if (character := data.get("character"))
                else None
            ),
            person=(
                CharacterBrief.from_dict(person)
                if (person := data.get("person"))
                else None
            ),
        )



def filter_none_parameters(params: dict) -> dict:
    return {key: value for key, value in params.items() if value is not None}


def handle_none_data(func):
    def wrapper(*args, **kwargs):
        data = args[1]

        if data is None:
            return None

        return func(*args, **kwargs)

    return wrapper



from codecs import open
from pathlib import Path

import tomli
from setuptools import setup, find_packages

with open("pyproject.toml", "rb") as f:
    pyproject = tomli.load(f)


def get_long_description() -> str:
    return (Path(__file__).parent / "README.md").read_text()


extra_requirements = {
    "readthedocs": [
        "furo",
        "readthedocs-sphinx-search",
        "Sphinx",
        "sphinx-hoverxref",
    ]
}

setup(
    name=pyproject["tool"]["poetry"]["name"],
    version=pyproject["tool"]["poetry"]["version"],
    packages=find_packages(exclude=["*.tests", "*.tests.*", "tests.*", "tests"]),
    extras_require=extra_requirements,
    install_requires=["aiohttp>=3.9.1", "tomli>=2.0.1", "setuptools>=69.0.3"],
    author="Snayt1k3",
    author_email="snayt1k3@gmail.com",
    description=pyproject["tool"]["poetry"]["description"],
    long_description_content_type="text/markdown",
    long_description=get_long_description(),
    url="https://github.com/Snayt1k3/shiki.py",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
    ],
)



from dataclasses import dataclass

from shikimori.utils.filter import handle_none_data


@dataclass
class Achievement:
    id: int
    neko_id: str
    level: int
    progress: int
    user_id: int
    created_at: str
    updated_at: str

    @classmethod
    @handle_none_data
    def from_dict(cls, data: dict):
        return cls(
            id=data.get("id"),
            neko_id=data.get("neko_id"),
            level=data.get("level"),
            progress=data.get("progress"),
            user_id=data.get("user_id"),
            created_at=data.get("created_at"),
            updated_at=data.get("updated_at"),
        )



import logging
import time

from .exceptions import TooManyRequests
from .request import Request
from shikimori.base import BaseLimiter

logger = logging.getLogger(__name__)

__all__ = ["RequestLimiter"]


class RequestLimiter(BaseLimiter):
    """Limiter for requests"""

    def __init__(self, max_requests_sec: int, max_requests_min: int, request: Request):
        self._max_requests_sec = max_requests_sec
        self._max_requests_min = max_requests_min

        self._current_requests_sec = 0
        self._current_requests_min = 0

        self._request = request
        self._last_request_time_minute = time.time()
        self._last_request_time_second = time.time()

    def _is_limit_exceeded(self):
        current_time = time.time()

        if current_time - self._last_request_time_second >= 1:
            self._current_requests_sec = 0
            self._last_request_time_second = current_time

        if current_time - self._last_request_time_minute >= 60:
            self._current_requests_min = 0
            self._last_request_time_minute = current_time

        return (
            self._current_requests_sec > self._max_requests_sec
            or self._current_requests_min > self._max_requests_min
        )

    async def make_request(self, method: str, **kwargs):
        """
        :param method: one of [GET, POST, PATCH, DELETE]
        :param kwargs: parameters for request
        :return: response or 'RequestError' or Raise 'TooManyRequests'
        """
        if self._is_limit_exceeded():

            logger.error("RATE LIMIT EXCEEDED")
            raise TooManyRequests("Rate limit exceeded")

        self._current_requests_min += 1
        self._current_requests_sec += 1

        logger.info(f"{method} Request to url - {kwargs['url']}")

        return await self._request.make_request(method, **kwargs)
